import datetime
import ipaddress


class Base:

    measurement_type = None

    def __init__(self, n, w=False):
        self.value = None
        self.name = n
        self.writeable = w

    def _from(self, v):
        return v

    def _to(self, v):
        return v

    def __repr__(self):
        return str(self.value)

    def __str__(self):
        return str(self.value)


class SelectionBase(Base):
    @property
    def options(self):
        return [v for k, v in self.codes.items()]

    def _to(self, v):
        if v in self.codes:
            return self.codes.get(v)

    def _from(self, v):
        for i, c in self.codes.items():
            if c == v:
                return i
        return None


class Celsius(Base):

    measurement_type = "celsius"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(float(v) * 10)


class Bool(Base):

    measurement_type = "boolean"

    def _to(self, v):
        return bool(v)

    def _from(self, v):
        return int(v)


class Frequency(Base):

    measurement_type = "Hz"


class Seconds(Base):

    measurement_type = "seconds"


class Pulses(Base):

    measurement_type = "pulses"


class IPAddress(Base):

    measurement_type = "ipaddress"

    def _to(self, v):
        if v<0:
            return str(ipaddress.IPv4Address(v + 2 ** 32))
        if v>2**32:
            return str(ipaddress.IPv4Address(v - 2 ** 32))
        return str(ipaddress.IPv4Address(v))

    def _from(self, v):
        result = int(ipaddress.IPv4Address(v))
        if result > 2 ** 32:
            return result - 2 ** 32
        return result

class Timestamp(Base):

    measurement_type = "timestamp"

    def _to(self, v):
        return datetime.datetime.fromtimestamp(v)

    def _from(self, v):
        return datetime.datetime.timestamp(v)


class Errorcode(Base):

    measurement_type = "errorcode"


class Kelvin(Base):

    measurement_type = "kelvin"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(v * 10)


class Pressure(Base):

    measurement_type = "bar"

    def _to(self, v):
        return v / 100

    def _from(self, v):
        return int(v * 100)


class Percent(Base):

    measurement_type = "percent"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(v * 10)


class Percent2(Base):

    measurement_type = "percent"

    def _to(self, v):
        return v

    def _from(self, v):
        return int(v)




class Speed(Base):

    measurement_type = "rpm"


class Power(Base):

    measurement_type = "W"


class Energy(Base):

    measurement_type = "energy"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(v * 10)


class Voltage(Base):

    measurement_type = "voltage"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(v * 10)


class Hours(Base):

    measurement_type = "hours"

    def _to(self, v):
        return v / 10

    def _from(self, v):
        return int(v * 10)


class Flow(Base):

    measurement_type = "flow"


class Level(Base):

    measurement_type = "level"


class Count(Base):

    measurement_type = "count"


class Version(Base):

    measurement_type = "version"

    def _to(self, v):
        return "".join([chr(c) for c in v]).strip("\x00")

    def _from(self):
        return None


class Icon(Base):

    measurement_type = "icon"


class HeatingMode(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "Automatic",
        1: "Second heatsource",
        2: "Party",
        3: "Holidays",
        4: "Off",
    }


class CoolingMode(SelectionBase):

    measurement_type = "selection"

    codes = {0: "Off", 1: "Automatic"}


class HotWaterMode(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "Automatic",
        1: "Second heatsource",
        2: "Party",
        3: "Holidays",
        4: "Off",
    }


class PoolMode(SelectionBase):

    measurement_type = "selection"

    codes = {0: "Automatic", 2: "Party", 3: "Holidays", 4: "Off"}


class MixedCircuitMode(SelectionBase):

    measurement_type = "selection"

    codes = {0: "Automatic", 2: "Party", 3: "Holidays", 4: "Off"}


class SolarMode(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "Automatic",
        1: "Second heatsource",
        2: "Party",
        3: "Holidays",
        4: "Off",
    }


class VentilationMode(SelectionBase):

    measurement_type = "selection"

    codes = {0: "Automatic", 1: "Party", 2: "Holidays", 3: "Off"}


class Code_WP(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "ERC",
        1: "SW1",
        2: "SW2",
        3: "WW1",
        4: "WW2",
        5: "L1I",
        6: "L2I",
        7: "L1A",
        8: "L2A",
        9: "KSW",
        10: "KLW",
        11: "SWC",
        12: "LWC",
        13: "L2G",
        14: "WZS",
        15: "L1I407",
        16: "L2I407",
        17: "L1A407",
        18: "L2A407",
        19: "L2G407",
        20: "LWC407",
        21: "L1AREV",
        22: "L2AREV",
        23: "WWC1",
        24: "WWC2",
        25: "L2G404",
        26: "WZW",
        27: "L1S",
        28: "L1H",
        29: "L2H",
        30: "WZWD",
        31: "ERC",
        40: "WWB_20",
        41: "LD5",
        42: "LD7",
        43: "SW 37_45",
        44: "SW 58_69",
        45: "SW 29_56",
        46: "LD5 (230V)",
        47: "LD7 (230 V)",
        48: "LD9",
        49: "LD5 REV",
        50: "LD7 REV",
        51: "LD5 REV 230V",
        52: "LD7 REV 230V",
        53: "LD9 REV 230V",
        54: "SW 291",
        55: "LW SEC",
        56: "HMD 2",
        57: "MSW 4",
        58: "MSW 6",
        59: "MSW 8",
        60: "MSW 10",
        61: "MSW 12",
        62: "MSW 14",
        63: "MSW 17",
        64: "MSW 19",
        65: "MSW 23",
        66: "MSW 26",
        67: "MSW 30",
        68: "MSW 4S",
        69: "MSW 6S",
        70: "MSW 8S",
        71: "MSW 10S",
        72: "MSW 13S",
        73: "MSW 16S",
        74: "MSW2-6S",
        75: "MSW4-16",
    }


class BivalenceLevel(SelectionBase):

    measurement_type = "selection"

    codes = {
        1: "one compressor allowed to run",
        2: "two compressors allowed to run",
        3: "additional compressor allowed to run",
    }


class OperationMode(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "heating",
        1: "hot water",
        2: "swimming pool/solar",
        3: "evu",
        4: "defrost",
        5: "no request",
        6: "heating external source",
        7: "cooling",
    }


class SwitchoffFile(SelectionBase):

    measurement_type = "selection"

    codes = {
        1: "heatpump error",
        2: "system error",
        3: "evu lock",
        4: "operation mode second heat generator",
        5: "air defrost",
        6: "maximal usage temprature",
        7: "minimal usage temperature",
        8: "lower usage limit",
        9: "no request",
    }


class MainMenuStatusLine1(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "heatpump running",
        1: "heatpump idle",
        2: "heatpump coming",
        3: "errorcode slot 0",
        4: "defrost",
        5: "witing on LIN connection",
        6: "compressor heating up",
        7: "pump forerun",
    }


class MainMenuStatusLine2(SelectionBase):

    measurement_type = "selection"

    codes = {0: "since", 1: "in"}


class MainMenuStatusLine3(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "heating",
        1: "no request",
        2: "grid switch on delay",
        3: "cycle lock",
        4: "lock time",
        5: "domestic water",
        6: "info bake out program",
        7: "defrost",
        8: "pump forerun",
        9: "thermal desinfection",
        10: "cooling",
        12: "swimming pool/solar",
        13: "heating external engery source",
        14: "domestic water external energy source",
        16: "flow monitoring",
        17: "second heat generator 1 active",
    }


class SecOperationMode(SelectionBase):

    measurement_type = "selection"

    codes = {
        0: "off",
        1: "cooling",
        2: "heating",
        3: "fault",
        4: "transition",
        5: "defrost",
        6: "waiting",
        7: "waiting",
        8: "transition",
        9: "stop",
        10: "manual",
        11: "simulation start",
        12: "evu lock",
    }


class Unknown(Base):

    measurement_type = None



# -*- coding: utf-8 -*-

import socket
import struct
import logging
from luxtronik.calculations import Calculations
from luxtronik.parameters import Parameters
from luxtronik.visibilities import Visibilities

logging.basicConfig(level="WARNING")
LOGGER = logging.getLogger("Luxtronik")


class Luxtronik:
    def __init__(self, host, port, safe=True):
        self._host = host
        self._port = port
        self._socket = None
        self.calculations = Calculations()
        self.parameters = Parameters(safe=safe)
        self.visibilities = Visibilities()
        self.read()

    def _connect(self):
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.connect((self._host, self._port))
        LOGGER.info(f"Connected to Luxtronik heatpump {self._host}:{self._port}")

    def _disconnect(self):
        self._socket.close()
        LOGGER.info(f"Disconnected from Luxtronik heatpump {self._host}:{self._port}")

    def read(self):
        self._connect()
        self._read_parameters()
        self._read_calculations()
        self._read_visibilities()
        self._disconnect()

    def write(self):
        self._connect()
        for id, value in self.parameters._queue.items():
            if not isinstance(id, int) or not isinstance(value, int):
                LOGGER.warn(f"Parameter id '{id}' or value '{value}' invalid!")
                continue
            LOGGER.info(f"Parameter '{id}' set to '{value}'")
            data = struct.pack(">iii", 3002, id, value)
            self._socket.sendall(data)
            cmd = struct.unpack(">i", self._socket.recv(4))[0]
            val = struct.unpack(">i", self._socket.recv(4))[0]
        self._disconnect()
        # flush queue after writing all values
        self.parameters._queue = {}

    def _read_parameters(self):
        data = []
        self._socket.sendall(struct.pack(">ii", 3003, 0))
        cmd = struct.unpack(">i", self._socket.recv(4))[0]
        len = struct.unpack(">i", self._socket.recv(4))[0]
        for i in range(0, len):
            data.append(struct.unpack(">i", self._socket.recv(4))[0])
        LOGGER.info(f"Read {len} parameters")
        self.parameters._parse(data)

    def _read_calculations(self):
        data = []
        self._socket.sendall(struct.pack(">ii", 3004, 0))
        cmd = struct.unpack(">i", self._socket.recv(4))[0]
        stat = struct.unpack(">i", self._socket.recv(4))[0]
        len = struct.unpack(">i", self._socket.recv(4))[0]
        for i in range(0, len):
            data.append(struct.unpack(">i", self._socket.recv(4))[0])
        LOGGER.info(f"Read {len} calculations")
        self.calculations._parse(data)

    def _read_visibilities(self):
        data = []
        self._socket.sendall(struct.pack(">ii", 3005, 0))
        cmd = struct.unpack(">i", self._socket.recv(4))[0]
        len = struct.unpack(">i", self._socket.recv(4))[0]
        for i in range(0, len):
            data.append(struct.unpack(">b", self._socket.recv(1))[0])
        LOGGER.info(f"Read {len} visibilities")
        self.visibilities._parse(data)



import logging

from luxtronik.datatypes import *

logging.basicConfig(level="WARNING")
LOGGER = logging.getLogger("Luxtronik.Parameters")


class Parameters:
    """Class that holds all parameters."""

    parameters = {
        0: Unknown("ID_Transfert_LuxNet"),
        1: Celsius("ID_Einst_WK_akt"),
        2: Celsius("ID_Einst_BWS_akt", True),
        3: HeatingMode("ID_Ba_Hz_akt", True),
        4: HotWaterMode("ID_Ba_Bw_akt", True),
        5: Unknown("ID_Ba_Al_akt"),
        6: Unknown("ID_SU_FrkdHz"),
        7: Unknown("ID_SU_FrkdBw"),
        8: Unknown("ID_SU_FrkdAl"),
        9: Unknown("ID_Einst_HReg_akt"),
        10: Unknown("ID_Einst_HzHwMAt_akt"),
        11: Unknown("ID_Einst_HzHwHKE_akt"),
        12: Unknown("ID_Einst_HzHKRANH_akt"),
        13: Unknown("ID_Einst_HzHKRABS_akt"),
        14: Unknown("ID_Einst_HzMK1E_akt"),
        15: Unknown("ID_Einst_HzMK1ANH_akt"),
        16: Unknown("ID_Einst_HzMK1ABS_akt"),
        17: Unknown("ID_Einst_HzFtRl_akt"),
        18: Unknown("ID_Einst_HzFtMK1Vl_akt"),
        19: Unknown("ID_Einst_SUBW_akt"),
        20: Unknown("ID_Einst_BwTDI_akt_MO"),
        21: Unknown("ID_Einst_BwTDI_akt_DI"),
        22: Unknown("ID_Einst_BwTDI_akt_MI"),
        23: Unknown("ID_Einst_BwTDI_akt_DO"),
        24: Unknown("ID_Einst_BwTDI_akt_FR"),
        25: Unknown("ID_Einst_BwTDI_akt_SA"),
        26: Unknown("ID_Einst_BwTDI_akt_SO"),
        27: Unknown("ID_Einst_BwTDI_akt_AL"),
        28: Unknown("ID_Einst_AnlKonf_akt"),
        29: Unknown("ID_Einst_Sprache_akt"),
        30: Unknown("ID_Switchoff_Zahler"),
        31: Unknown("ID_Switchoff_index"),
        32: Unknown("ID_Einst_EvuTyp_akt"),
        33: Unknown("ID_Einst_RFVEinb_akt"),
        34: Unknown("ID_Einst_AbtZykMax_akt"),
        35: Unknown("ID_Einst_HREinb_akt"),
        36: Unknown("ID_Einst_ZWE1Art_akt"),
        37: Unknown("ID_Einst_ZWE1Fkt_akt"),
        38: Unknown("ID_Einst_ZWE2Art_akt"),
        39: Unknown("ID_Einst_ZWE2Fkt_akt"),
        40: Unknown("ID_Einst_BWBer_akt"),
        41: Unknown("ID_Einst_En_Inst"),
        42: Unknown("ID_Einst_MK1Typ_akt"),
        43: Unknown("ID_Einst_ABTLuft_akt"),
        44: Unknown("ID_Einst_TLAbt_akt"),
        45: Unknown("ID_Einst_LAbtTime_akt"),
        46: Unknown("ID_Einst_ASDTyp_akt"),
        47: Unknown("ID_Einst_LGST_akt"),
        48: Unknown("ID_Einst_BwWpTime_akt"),
        49: Unknown("ID_Einst_Popt_akt"),
        50: Unknown("ID_Einst_Kurzprog_akt"),
        51: Unknown("ID_Timer_Kurzprog_akt"),
        52: Unknown("ID_Einst_ManAbt_akt"),
        53: Unknown("ID_Einst_Ahz_akt"),
        54: Unknown("ID_Einst_TVL_Ahz_1"),
        55: Unknown("ID_Einst_TVL_Ahz_2"),
        56: Unknown("ID_Einst_TVL_Ahz_3"),
        57: Unknown("ID_Einst_TVL_Ahz_4"),
        58: Unknown("ID_Einst_TVL_Ahz_5"),
        59: Unknown("ID_Einst_TVL_Ahz_6"),
        60: Unknown("ID_Einst_TVL_Ahz_7"),
        61: Unknown("ID_Einst_TVL_Ahz_8"),
        62: Unknown("ID_Einst_TVL_Ahz_9"),
        63: Unknown("ID_Einst_TVL_Ahz_10"),
        64: Unknown("ID_Einst_TVL_Std_1"),
        65: Unknown("ID_Einst_TVL_Std_2"),
        66: Unknown("ID_Einst_TVL_Std_3"),
        67: Unknown("ID_Einst_TVL_Std_4"),
        68: Unknown("ID_Einst_TVL_Std_5"),
        69: Unknown("ID_Einst_TVL_Std_6"),
        70: Unknown("ID_Einst_TVL_Std_7"),
        71: Unknown("ID_Einst_TVL_Std_8"),
        72: Unknown("ID_Einst_TVL_Std_9"),
        73: Unknown("ID_Einst_TVL_Std_10"),
        74: Unknown("ID_Einst_BWS_Hyst_akt"),
        75: Unknown("ID_Temp_TBW_BwHD_saved"),
        76: Unknown("ID_Einst_ABT1_akt"),
        77: Unknown("ID_Einst_LABTpaus_akt"),
        78: Unknown("ID_AHZ_state_akt"),
        79: Celsius("ID_Sollwert_TRL_HZ_AHZ", True),
        80: Unknown("ID_AHP_valid_records"),
        81: Unknown("ID_Timer_AHZ_akt"),
        82: Unknown("ID_Einst_BWTINP_akt"),
        83: Unknown("ID_Einst_ZUPTYP_akt"),
        84: Unknown("ID_Sollwert_TLG_max"),
        85: Unknown("ID_Einst_BWZIP_akt"),
        86: Unknown("ID_Einst_ERRmZWE_akt"),
        87: Unknown("ID_Einst_TRBegr_akt"),
        88: Unknown("ID_Einst_HRHyst_akt"),
        89: Unknown("ID_Einst_TRErhmax_akt"),
        90: Unknown("ID_Einst_ZWEFreig_akt"),
        91: Unknown("ID_Einst_TAmax_akt"),
        92: Unknown("ID_Einst_TAmin_akt"),
        93: Unknown("ID_Einst_TWQmin_akt"),
        94: Unknown("ID_Einst_THGmax_akt"),
        95: Unknown("ID_Einst_FRGT2VD_akt"),
        96: Unknown("ID_Einst_TV2VDBW_akt"),
        97: Unknown("ID_Einst_SuAll_akt"),
        98: Unknown("ID_Einst_TAbtEnd_akt"),
        99: Unknown("ID_Einst_NrKlingel_akt"),
        100: Unknown("ID_Einst_BWStyp_akt"),
        101: Unknown("ID_Einst_ABT2_akt"),
        102: Unknown("ID_Einst_UeVd_akt"),
        103: Unknown("ID_Einst_RTyp_akt"),
        104: Unknown("ID_Einst_AhpM_akt"),
        105: Celsius("ID_Soll_BWS_akt", True),
        106: Unknown("ID_Timer_Password"),
        107: Unknown("ID_Einst_Zugangscode"),
        108: CoolingMode("ID_Einst_BA_Kuehl_akt", True),
        109: Unknown("ID_Sollwert_Kuehl1_akt"),
        110: Celsius("ID_Einst_KuehlFreig_akt", True),
        111: Unknown("ID_Einst_TAbsMin_akt"),
        112: Unknown("ID_TWQmin_saved"),
        113: Unknown("ID_CWP_saved"),
        114: Unknown("ID_Einst_Anode_akt"),
        115: Unknown("ID_Timer_pexoff_akt"),
        116: Unknown("ID_Einst_AnlPrio_Hzakt"),
        117: Unknown("ID_Einst_AnlPrio_Bwakt"),
        118: Unknown("ID_Einst_AnlPrio_Swakt"),
        119: PoolMode("ID_Ba_Sw_akt", True),
        120: Unknown("ID_Einst_RTypMK1_akt"),
        121: Unknown("ID_Einst_RTypMK2_akt"),
        122: Unknown("ID_Einst_TDC_Ein_akt"),
        123: Unknown("ID_Einst_TDC_Aus_akt"),
        124: Celsius("ID_Einst_TDC_Max_akt", True),
        125: Unknown("ID_Einst_HysHzExEn_akt"),
        126: Unknown("ID_Einst_HysBwExEn_akt"),
        127: Unknown("ID_Einst_ZWE3Art_akt"),
        128: Unknown("ID_Einst_ZWE3Fkt_akt"),
        129: Unknown("ID_Einst_HzSup_akt"),
        130: Unknown("ID_Einst_MK2Typ_akt"),
        131: Unknown("ID_Einst_KuTyp_akt"),
        132: Celsius("ID_Sollwert_KuCft1_akt", True),
        133: Celsius("ID_Sollwert_KuCft2_akt", True),
        134: Celsius("ID_Sollwert_AtDif1_akt", True),
        135: Celsius("ID_Sollwert_AtDif2_akt", True),
        136: Unknown("ID_SU_FrkdSwb"),
        137: Unknown("ID_Einst_SwbBer_akt"),
        138: Unknown("ID_Einst_TV2VDSWB_akt"),
        139: Unknown("ID_Einst_MinSwan_Time_akt"),
        140: Unknown("ID_Einst_SuMk2_akt"),
        141: Unknown("ID_Einst_HzMK2E_akt"),
        142: Unknown("ID_Einst_HzMK2ANH_akt"),
        143: Unknown("ID_Einst_HzMK2ABS_akt"),
        144: Unknown("ID_Einst_HzMK2Hgr_akt"),
        145: Unknown("ID_Einst_HzFtMK2Vl_akt"),
        146: Unknown("ID_Temp_THG_BwHD_saved"),
        147: Unknown("ID_Temp_TA_BwHD_saved"),
        148: Unknown("ID_Einst_BwHup_akt"),
        149: Unknown("ID_Einst_TVLmax_akt"),
        150: Unknown("ID_Einst_MK1LzFaktor_akt"),
        151: Unknown("ID_Einst_MK2LzFaktor_akt"),
        152: Unknown("ID_Einst_MK1PerFaktor_akt"),
        153: Unknown("ID_Einst_MK2PerFaktor_akt"),
        154: Unknown("ID_Entl_Zyklus_akt"),
        155: Unknown("ID_Einst_Entl_time_akt"),
        156: Unknown("ID_Entl_Pause"),
        157: Unknown("ID_Entl_timer"),
        158: Unknown("ID_Einst_Entl_akt"),
        159: Unknown("ID_Ahz_HLeist_confirmed"),
        160: Unknown("ID_FirstInit_akt"),
        161: Unknown("ID_Einst_SuAll_akt2"),
        162: Unknown("ID_Einst_SuAllWo_zeit_0_0"),
        163: Unknown("ID_Einst_SuAllWo_zeit_0_1"),
        164: Unknown("ID_Einst_SuAllWo_zeit_1_0"),
        165: Unknown("ID_Einst_SuAllWo_zeit_1_1"),
        166: Unknown("ID_Einst_SuAllWo_zeit_2_0"),
        167: Unknown("ID_Einst_SuAllWo_zeit_2_1"),
        168: Unknown("ID_Einst_SuAll25_zeit_0_0"),
        169: Unknown("ID_Einst_SuAll25_zeit_0_1"),
        170: Unknown("ID_Einst_SuAll25_zeit_1_0"),
        171: Unknown("ID_Einst_SuAll25_zeit_1_1"),
        172: Unknown("ID_Einst_SuAll25_zeit_2_0"),
        173: Unknown("ID_Einst_SuAll25_zeit_2_1"),
        174: Unknown("ID_Einst_SuAll25_zeit_0_2"),
        175: Unknown("ID_Einst_SuAll25_zeit_0_3"),
        176: Unknown("ID_Einst_SuAll25_zeit_1_2"),
        177: Unknown("ID_Einst_SuAll25_zeit_1_3"),
        178: Unknown("ID_Einst_SuAll25_zeit_2_2"),
        179: Unknown("ID_Einst_SuAll25_zeit_2_3"),
        180: Unknown("ID_Einst_SuAllTg_zeit_0_0"),
        181: Unknown("ID_Einst_SuAllTg_zeit_0_1"),
        182: Unknown("ID_Einst_SuAllTg_zeit_1_0"),
        183: Unknown("ID_Einst_SuAllTg_zeit_1_1"),
        184: Unknown("ID_Einst_SuAllTg_zeit_2_0"),
        185: Unknown("ID_Einst_SuAllTg_zeit_2_1"),
        186: Unknown("ID_Einst_SuAllTg_zeit_0_2"),
        187: Unknown("ID_Einst_SuAllTg_zeit_0_3"),
        188: Unknown("ID_Einst_SuAllTg_zeit_1_2"),
        189: Unknown("ID_Einst_SuAllTg_zeit_1_3"),
        190: Unknown("ID_Einst_SuAllTg_zeit_2_2"),
        191: Unknown("ID_Einst_SuAllTg_zeit_2_3"),
        192: Unknown("ID_Einst_SuAllTg_zeit_0_4"),
        193: Unknown("ID_Einst_SuAllTg_zeit_0_5"),
        194: Unknown("ID_Einst_SuAllTg_zeit_1_4"),
        195: Unknown("ID_Einst_SuAllTg_zeit_1_5"),
        196: Unknown("ID_Einst_SuAllTg_zeit_2_4"),
        197: Unknown("ID_Einst_SuAllTg_zeit_2_5"),
        198: Unknown("ID_Einst_SuAllTg_zeit_0_6"),
        199: Unknown("ID_Einst_SuAllTg_zeit_0_7"),
        200: Unknown("ID_Einst_SuAllTg_zeit_1_6"),
        201: Unknown("ID_Einst_SuAllTg_zeit_1_7"),
        202: Unknown("ID_Einst_SuAllTg_zeit_2_6"),
        203: Unknown("ID_Einst_SuAllTg_zeit_2_7"),
        204: Unknown("ID_Einst_SuAllTg_zeit_0_8"),
        205: Unknown("ID_Einst_SuAllTg_zeit_0_9"),
        206: Unknown("ID_Einst_SuAllTg_zeit_1_8"),
        207: Unknown("ID_Einst_SuAllTg_zeit_1_9"),
        208: Unknown("ID_Einst_SuAllTg_zeit_2_8"),
        209: Unknown("ID_Einst_SuAllTg_zeit_2_9"),
        210: Unknown("ID_Einst_SuAllTg_zeit_0_10"),
        211: Unknown("ID_Einst_SuAllTg_zeit_0_11"),
        212: Unknown("ID_Einst_SuAllTg_zeit_1_10"),
        213: Unknown("ID_Einst_SuAllTg_zeit_1_11"),
        214: Unknown("ID_Einst_SuAllTg_zeit_2_10"),
        215: Unknown("ID_Einst_SuAllTg_zeit_2_11"),
        216: Unknown("ID_Einst_SuAllTg_zeit_0_12"),
        217: Unknown("ID_Einst_SuAllTg_zeit_0_13"),
        218: Unknown("ID_Einst_SuAllTg_zeit_1_12"),
        219: Unknown("ID_Einst_SuAllTg_zeit_1_13"),
        220: Unknown("ID_Einst_SuAllTg_zeit_2_12"),
        221: Unknown("ID_Einst_SuAllTg_zeit_2_13"),
        222: Unknown("ID_Einst_SuHkr_akt"),
        223: Unknown("ID_Einst_SuHkrW0_zeit_0_0"),
        224: Unknown("ID_Einst_SuHkrW0_zeit_0_1"),
        225: Unknown("ID_Einst_SuHkrW0_zeit_1_0"),
        226: Unknown("ID_Einst_SuHkrW0_zeit_1_1"),
        227: Unknown("ID_Einst_SuHkrW0_zeit_2_0"),
        228: Unknown("ID_Einst_SuHkrW0_zeit_2_1"),
        229: Unknown("ID_Einst_SuHkr25_zeit_0_0"),
        230: Unknown("ID_Einst_SuHkr25_zeit_0_1"),
        231: Unknown("ID_Einst_SuHkr25_zeit_1_0"),
        232: Unknown("ID_Einst_SuHkr25_zeit_1_1"),
        233: Unknown("ID_Einst_SuHkr25_zeit_2_0"),
        234: Unknown("ID_Einst_SuHkr25_zeit_2_1"),
        235: Unknown("ID_Einst_SuHkr25_zeit_0_2"),
        236: Unknown("ID_Einst_SuHkr25_zeit_0_3"),
        237: Unknown("ID_Einst_SuHkr25_zeit_1_2"),
        238: Unknown("ID_Einst_SuHkr25_zeit_1_3"),
        239: Unknown("ID_Einst_SuHkr25_zeit_2_2"),
        240: Unknown("ID_Einst_SuHkr25_zeit_2_3"),
        241: Unknown("ID_Einst_SuHkrTG_zeit_0_0"),
        242: Unknown("ID_Einst_SuHkrTG_zeit_0_1"),
        243: Unknown("ID_Einst_SuHkrTG_zeit_1_0"),
        244: Unknown("ID_Einst_SuHkrTG_zeit_1_1"),
        245: Unknown("ID_Einst_SuHkrTG_zeit_2_0"),
        246: Unknown("ID_Einst_SuHkrTG_zeit_2_1"),
        247: Unknown("ID_Einst_SuHkrTG_zeit_0_2"),
        248: Unknown("ID_Einst_SuHkrTG_zeit_0_3"),
        249: Unknown("ID_Einst_SuHkrTG_zeit_1_2"),
        250: Unknown("ID_Einst_SuHkrTG_zeit_1_3"),
        251: Unknown("ID_Einst_SuHkrTG_zeit_2_2"),
        252: Unknown("ID_Einst_SuHkrTG_zeit_2_3"),
        253: Unknown("ID_Einst_SuHkrTG_zeit_0_4"),
        254: Unknown("ID_Einst_SuHkrTG_zeit_0_5"),
        255: Unknown("ID_Einst_SuHkrTG_zeit_1_4"),
        256: Unknown("ID_Einst_SuHkrTG_zeit_1_5"),
        257: Unknown("ID_Einst_SuHkrTG_zeit_2_4"),
        258: Unknown("ID_Einst_SuHkrTG_zeit_2_5"),
        259: Unknown("ID_Einst_SuHkrTG_zeit_0_6"),
        260: Unknown("ID_Einst_SuHkrTG_zeit_0_7"),
        261: Unknown("ID_Einst_SuHkrTG_zeit_1_6"),
        262: Unknown("ID_Einst_SuHkrTG_zeit_1_7"),
        263: Unknown("ID_Einst_SuHkrTG_zeit_2_6"),
        264: Unknown("ID_Einst_SuHkrTG_zeit_2_7"),
        265: Unknown("ID_Einst_SuHkrTG_zeit_0_8"),
        266: Unknown("ID_Einst_SuHkrTG_zeit_0_9"),
        267: Unknown("ID_Einst_SuHkrTG_zeit_1_8"),
        268: Unknown("ID_Einst_SuHkrTG_zeit_1_9"),
        269: Unknown("ID_Einst_SuHkrTG_zeit_2_8"),
        270: Unknown("ID_Einst_SuHkrTG_zeit_2_9"),
        271: Unknown("ID_Einst_SuHkrTG_zeit_0_10"),
        272: Unknown("ID_Einst_SuHkrTG_zeit_0_11"),
        273: Unknown("ID_Einst_SuHkrTG_zeit_1_10"),
        274: Unknown("ID_Einst_SuHkrTG_zeit_1_11"),
        275: Unknown("ID_Einst_SuHkrTG_zeit_2_10"),
        276: Unknown("ID_Einst_SuHkrTG_zeit_2_11"),
        277: Unknown("ID_Einst_SuHkrTG_zeit_0_12"),
        278: Unknown("ID_Einst_SuHkrTG_zeit_0_13"),
        279: Unknown("ID_Einst_SuHkrTG_zeit_1_12"),
        280: Unknown("ID_Einst_SuHkrTG_zeit_1_13"),
        281: Unknown("ID_Einst_SuHkrTG_zeit_2_12"),
        282: Unknown("ID_Einst_SuHkrTG_zeit_2_13"),
        283: Unknown("ID_Einst_SuMk1_akt"),
        284: Unknown("ID_Einst_SuMk1W0_zeit_0_0"),
        285: Unknown("ID_Einst_SuMk1W0_zeit_0_1"),
        286: Unknown("ID_Einst_SuMk1W0_zeit_1_0"),
        287: Unknown("ID_Einst_SuMk1W0_zeit_1_1"),
        288: Unknown("ID_Einst_SuMk1W0_zeit_2_0"),
        289: Unknown("ID_Einst_SuMk1W0_zeit_2_1"),
        290: Unknown("ID_Einst_SuMk125_zeit_0_0"),
        291: Unknown("ID_Einst_SuMk125_zeit_0_1"),
        292: Unknown("ID_Einst_SuMk125_zeit_1_0"),
        293: Unknown("ID_Einst_SuMk125_zeit_1_1"),
        294: Unknown("ID_Einst_SuMk125_zeit_2_0"),
        295: Unknown("ID_Einst_SuMk125_zeit_2_1"),
        296: Unknown("ID_Einst_SuMk125_zeit_0_2"),
        297: Unknown("ID_Einst_SuMk125_zeit_0_3"),
        298: Unknown("ID_Einst_SuMk125_zeit_1_2"),
        299: Unknown("ID_Einst_SuMk125_zeit_1_3"),
        300: Unknown("ID_Einst_SuMk125_zeit_2_2"),
        301: Unknown("ID_Einst_SuMk125_zeit_2_3"),
        302: Unknown("ID_Einst_SuMk1TG_zeit_0_0"),
        303: Unknown("ID_Einst_SuMk1TG_zeit_0_1"),
        304: Unknown("ID_Einst_SuMk1TG_zeit_1_0"),
        305: Unknown("ID_Einst_SuMk1TG_zeit_1_1"),
        306: Unknown("ID_Einst_SuMk1TG_zeit_2_0"),
        307: Unknown("ID_Einst_SuMk1TG_zeit_2_1"),
        308: Unknown("ID_Einst_SuMk1TG_zeit_0_2"),
        309: Unknown("ID_Einst_SuMk1TG_zeit_0_3"),
        310: Unknown("ID_Einst_SuMk1TG_zeit_1_2"),
        311: Unknown("ID_Einst_SuMk1TG_zeit_1_3"),
        312: Unknown("ID_Einst_SuMk1TG_zeit_2_2"),
        313: Unknown("ID_Einst_SuMk1TG_zeit_2_3"),
        314: Unknown("ID_Einst_SuMk1TG_zeit_0_4"),
        315: Unknown("ID_Einst_SuMk1TG_zeit_0_5"),
        316: Unknown("ID_Einst_SuMk1TG_zeit_1_4"),
        317: Unknown("ID_Einst_SuMk1TG_zeit_1_5"),
        318: Unknown("ID_Einst_SuMk1TG_zeit_2_4"),
        319: Unknown("ID_Einst_SuMk1TG_zeit_2_5"),
        320: Unknown("ID_Einst_SuMk1TG_zeit_0_6"),
        321: Unknown("ID_Einst_SuMk1TG_zeit_0_7"),
        322: Unknown("ID_Einst_SuMk1TG_zeit_1_6"),
        323: Unknown("ID_Einst_SuMk1TG_zeit_1_7"),
        324: Unknown("ID_Einst_SuMk1TG_zeit_2_6"),
        325: Unknown("ID_Einst_SuMk1TG_zeit_2_7"),
        326: Unknown("ID_Einst_SuMk1TG_zeit_0_8"),
        327: Unknown("ID_Einst_SuMk1TG_zeit_0_9"),
        328: Unknown("ID_Einst_SuMk1TG_zeit_1_8"),
        329: Unknown("ID_Einst_SuMk1TG_zeit_1_9"),
        330: Unknown("ID_Einst_SuMk1TG_zeit_2_8"),
        331: Unknown("ID_Einst_SuMk1TG_zeit_2_9"),
        332: Unknown("ID_Einst_SuMk1TG_zeit_0_10"),
        333: Unknown("ID_Einst_SuMk1TG_zeit_0_11"),
        334: Unknown("ID_Einst_SuMk1TG_zeit_1_10"),
        335: Unknown("ID_Einst_SuMk1TG_zeit_1_11"),
        336: Unknown("ID_Einst_SuMk1TG_zeit_2_10"),
        337: Unknown("ID_Einst_SuMk1TG_zeit_2_11"),
        338: Unknown("ID_Einst_SuMk1TG_zeit_0_12"),
        339: Unknown("ID_Einst_SuMk1TG_zeit_0_13"),
        340: Unknown("ID_Einst_SuMk1TG_zeit_1_12"),
        341: Unknown("ID_Einst_SuMk1TG_zeit_1_13"),
        342: Unknown("ID_Einst_SuMk1TG_zeit_2_12"),
        343: Unknown("ID_Einst_SuMk1TG_zeit_2_13"),
        344: Unknown("ID_Einst_SuMk2_akt2"),
        345: Unknown("ID_Einst_SuMk2Wo_zeit_0_0"),
        346: Unknown("ID_Einst_SuMk2Wo_zeit_0_1"),
        347: Unknown("ID_Einst_SuMk2Wo_zeit_1_0"),
        348: Unknown("ID_Einst_SuMk2Wo_zeit_1_1"),
        349: Unknown("ID_Einst_SuMk2Wo_zeit_2_0"),
        350: Unknown("ID_Einst_SuMk2Wo_zeit_2_1"),
        351: Unknown("ID_Einst_SuMk225_zeit_0_0"),
        352: Unknown("ID_Einst_SuMk225_zeit_0_1"),
        353: Unknown("ID_Einst_SuMk225_zeit_1_0"),
        354: Unknown("ID_Einst_SuMk225_zeit_1_1"),
        355: Unknown("ID_Einst_SuMk225_zeit_2_0"),
        356: Unknown("ID_Einst_SuMk225_zeit_2_1"),
        357: Unknown("ID_Einst_SuMk225_zeit_0_2"),
        358: Unknown("ID_Einst_SuMk225_zeit_0_3"),
        359: Unknown("ID_Einst_SuMk225_zeit_1_2"),
        360: Unknown("ID_Einst_SuMk225_zeit_1_3"),
        361: Unknown("ID_Einst_SuMk225_zeit_2_2"),
        362: Unknown("ID_Einst_SuMk225_zeit_2_3"),
        363: Unknown("ID_Einst_SuMk2Tg_zeit_0_0"),
        364: Unknown("ID_Einst_SuMk2Tg_zeit_0_1"),
        365: Unknown("ID_Einst_SuMk2Tg_zeit_1_0"),
        366: Unknown("ID_Einst_SuMk2Tg_zeit_1_1"),
        367: Unknown("ID_Einst_SuMk2Tg_zeit_2_0"),
        368: Unknown("ID_Einst_SuMk2Tg_zeit_2_1"),
        369: Unknown("ID_Einst_SuMk2Tg_zeit_0_2"),
        370: Unknown("ID_Einst_SuMk2Tg_zeit_0_3"),
        371: Unknown("ID_Einst_SuMk2Tg_zeit_1_2"),
        372: Unknown("ID_Einst_SuMk2Tg_zeit_1_3"),
        373: Unknown("ID_Einst_SuMk2Tg_zeit_2_2"),
        374: Unknown("ID_Einst_SuMk2Tg_zeit_2_3"),
        375: Unknown("ID_Einst_SuMk2Tg_zeit_0_4"),
        376: Unknown("ID_Einst_SuMk2Tg_zeit_0_5"),
        377: Unknown("ID_Einst_SuMk2Tg_zeit_1_4"),
        378: Unknown("ID_Einst_SuMk2Tg_zeit_1_5"),
        379: Unknown("ID_Einst_SuMk2Tg_zeit_2_4"),
        380: Unknown("ID_Einst_SuMk2Tg_zeit_2_5"),
        381: Unknown("ID_Einst_SuMk2Tg_zeit_0_6"),
        382: Unknown("ID_Einst_SuMk2Tg_zeit_0_7"),
        383: Unknown("ID_Einst_SuMk2Tg_zeit_1_6"),
        384: Unknown("ID_Einst_SuMk2Tg_zeit_1_7"),
        385: Unknown("ID_Einst_SuMk2Tg_zeit_2_6"),
        386: Unknown("ID_Einst_SuMk2Tg_zeit_2_7"),
        387: Unknown("ID_Einst_SuMk2Tg_zeit_0_8"),
        388: Unknown("ID_Einst_SuMk2Tg_zeit_0_9"),
        389: Unknown("ID_Einst_SuMk2Tg_zeit_1_8"),
        390: Unknown("ID_Einst_SuMk2Tg_zeit_1_9"),
        391: Unknown("ID_Einst_SuMk2Tg_zeit_2_8"),
        392: Unknown("ID_Einst_SuMk2Tg_zeit_2_9"),
        393: Unknown("ID_Einst_SuMk2Tg_zeit_0_10"),
        394: Unknown("ID_Einst_SuMk2Tg_zeit_0_11"),
        395: Unknown("ID_Einst_SuMk2Tg_zeit_1_10"),
        396: Unknown("ID_Einst_SuMk2Tg_zeit_1_11"),
        397: Unknown("ID_Einst_SuMk2Tg_zeit_2_10"),
        398: Unknown("ID_Einst_SuMk2Tg_zeit_2_11"),
        399: Unknown("ID_Einst_SuMk2Tg_zeit_0_12"),
        400: Unknown("ID_Einst_SuMk2Tg_zeit_0_13"),
        401: Unknown("ID_Einst_SuMk2Tg_zeit_1_12"),
        402: Unknown("ID_Einst_SuMk2Tg_zeit_1_13"),
        403: Unknown("ID_Einst_SuMk2Tg_zeit_2_12"),
        404: Unknown("ID_Einst_SuMk2Tg_zeit_2_13"),
        405: Unknown("ID_Einst_SUBW_akt2"),
        406: Unknown("ID_Einst_SuBwWO_zeit_0_0"),
        407: Unknown("ID_Einst_SuBwWO_zeit_0_1"),
        408: Unknown("ID_Einst_SuBwWO_zeit_1_0"),
        409: Unknown("ID_Einst_SuBwWO_zeit_1_1"),
        410: Unknown("ID_Einst_SuBwWO_zeit_2_0"),
        411: Unknown("ID_Einst_SuBwWO_zeit_2_1"),
        412: Unknown("ID_Einst_SuBwWO_zeit_3_0"),
        413: Unknown("ID_Einst_SuBwWO_zeit_3_1"),
        414: Unknown("ID_Einst_SuBwWO_zeit_4_0"),
        415: Unknown("ID_Einst_SuBwWO_zeit_4_1"),
        416: Unknown("ID_Einst_SuBw25_zeit_0_0"),
        417: Unknown("ID_Einst_SuBw25_zeit_0_1"),
        418: Unknown("ID_Einst_SuBw25_zeit_1_0"),
        419: Unknown("ID_Einst_SuBw25_zeit_1_1"),
        420: Unknown("ID_Einst_SuBw25_zeit_2_0"),
        421: Unknown("ID_Einst_SuBw25_zeit_2_1"),
        422: Unknown("ID_Einst_SuBw25_zeit_3_0"),
        423: Unknown("ID_Einst_SuBw25_zeit_3_1"),
        424: Unknown("ID_Einst_SuBw25_zeit_4_0"),
        425: Unknown("ID_Einst_SuBw25_zeit_4_1"),
        426: Unknown("ID_Einst_SuBw25_zeit_0_2"),
        427: Unknown("ID_Einst_SuBw25_zeit_0_3"),
        428: Unknown("ID_Einst_SuBw25_zeit_1_2"),
        429: Unknown("ID_Einst_SuBw25_zeit_1_3"),
        430: Unknown("ID_Einst_SuBw25_zeit_2_2"),
        431: Unknown("ID_Einst_SuBw25_zeit_2_3"),
        432: Unknown("ID_Einst_SuBw25_zeit_3_2"),
        433: Unknown("ID_Einst_SuBw25_zeit_3_3"),
        434: Unknown("ID_Einst_SuBw25_zeit_4_2"),
        435: Unknown("ID_Einst_SuBw25_zeit_4_3"),
        436: Unknown("ID_Einst_SuBwTG_zeit_0_0"),
        437: Unknown("ID_Einst_SuBwTG_zeit_0_1"),
        438: Unknown("ID_Einst_SuBwTG_zeit_1_0"),
        439: Unknown("ID_Einst_SuBwTG_zeit_1_1"),
        440: Unknown("ID_Einst_SuBwTG_zeit_2_0"),
        441: Unknown("ID_Einst_SuBwTG_zeit_2_1"),
        442: Unknown("ID_Einst_SuBwTG_zeit_3_0"),
        443: Unknown("ID_Einst_SuBwTG_zeit_3_1"),
        444: Unknown("ID_Einst_SuBwTG_zeit_4_0"),
        445: Unknown("ID_Einst_SuBwTG_zeit_4_1"),
        446: Unknown("ID_Einst_SuBwTG_zeit_0_2"),
        447: Unknown("ID_Einst_SuBwTG_zeit_0_3"),
        448: Unknown("ID_Einst_SuBwTG_zeit_1_2"),
        449: Unknown("ID_Einst_SuBwTG_zeit_1_3"),
        450: Unknown("ID_Einst_SuBwTG_zeit_2_2"),
        451: Unknown("ID_Einst_SuBwTG_zeit_2_3"),
        452: Unknown("ID_Einst_SuBwTG_zeit_3_2"),
        453: Unknown("ID_Einst_SuBwTG_zeit_3_3"),
        454: Unknown("ID_Einst_SuBwTG_zeit_4_2"),
        455: Unknown("ID_Einst_SuBwTG_zeit_4_3"),
        456: Unknown("ID_Einst_SuBwTG_zeit_0_4"),
        457: Unknown("ID_Einst_SuBwTG_zeit_0_5"),
        458: Unknown("ID_Einst_SuBwTG_zeit_1_4"),
        459: Unknown("ID_Einst_SuBwTG_zeit_1_5"),
        460: Unknown("ID_Einst_SuBwTG_zeit_2_4"),
        461: Unknown("ID_Einst_SuBwTG_zeit_2_5"),
        462: Unknown("ID_Einst_SuBwTG_zeit_3_4"),
        463: Unknown("ID_Einst_SuBwTG_zeit_3_5"),
        464: Unknown("ID_Einst_SuBwTG_zeit_4_4"),
        465: Unknown("ID_Einst_SuBwTG_zeit_4_5"),
        466: Unknown("ID_Einst_SuBwTG_zeit_0_6"),
        467: Unknown("ID_Einst_SuBwTG_zeit_0_7"),
        468: Unknown("ID_Einst_SuBwTG_zeit_1_6"),
        469: Unknown("ID_Einst_SuBwTG_zeit_1_7"),
        470: Unknown("ID_Einst_SuBwTG_zeit_2_6"),
        471: Unknown("ID_Einst_SuBwTG_zeit_2_7"),
        472: Unknown("ID_Einst_SuBwTG_zeit_3_6"),
        473: Unknown("ID_Einst_SuBwTG_zeit_3_7"),
        474: Unknown("ID_Einst_SuBwTG_zeit_4_6"),
        475: Unknown("ID_Einst_SuBwTG_zeit_4_7"),
        476: Unknown("ID_Einst_SuBwTG_zeit_0_8"),
        477: Unknown("ID_Einst_SuBwTG_zeit_0_9"),
        478: Unknown("ID_Einst_SuBwTG_zeit_1_8"),
        479: Unknown("ID_Einst_SuBwTG_zeit_1_9"),
        480: Unknown("ID_Einst_SuBwTG_zeit_2_8"),
        481: Unknown("ID_Einst_SuBwTG_zeit_2_9"),
        482: Unknown("ID_Einst_SuBwTG_zeit_3_8"),
        483: Unknown("ID_Einst_SuBwTG_zeit_3_9"),
        484: Unknown("ID_Einst_SuBwTG_zeit_4_8"),
        485: Unknown("ID_Einst_SuBwTG_zeit_4_9"),
        486: Unknown("ID_Einst_SuBwTG_zeit_0_10"),
        487: Unknown("ID_Einst_SuBwTG_zeit_0_11"),
        488: Unknown("ID_Einst_SuBwTG_zeit_1_10"),
        489: Unknown("ID_Einst_SuBwTG_zeit_1_11"),
        490: Unknown("ID_Einst_SuBwTG_zeit_2_10"),
        491: Unknown("ID_Einst_SuBwTG_zeit_2_11"),
        492: Unknown("ID_Einst_SuBwTG_zeit_3_10"),
        493: Unknown("ID_Einst_SuBwTG_zeit_3_11"),
        494: Unknown("ID_Einst_SuBwTG_zeit_4_10"),
        495: Unknown("ID_Einst_SuBwTG_zeit_4_11"),
        496: Unknown("ID_Einst_SuBwTG_zeit_0_12"),
        497: Unknown("ID_Einst_SuBwTG_zeit_0_13"),
        498: Unknown("ID_Einst_SuBwTG_zeit_1_12"),
        499: Unknown("ID_Einst_SuBwTG_zeit_1_13"),
        500: Unknown("ID_Einst_SuBwTG_zeit_2_12"),
        501: Unknown("ID_Einst_SuBwTG_zeit_2_13"),
        502: Unknown("ID_Einst_SuBwTG_zeit_3_12"),
        503: Unknown("ID_Einst_SuBwTG_zeit_3_13"),
        504: Unknown("ID_Einst_SuBwTG_zeit_4_12"),
        505: Unknown("ID_Einst_SuBwTG_zeit_4_13"),
        506: Unknown("ID_Einst_SuZIP_akt"),
        507: Unknown("ID_Einst_SuZIPWo_zeit_0_0"),
        508: Unknown("ID_Einst_SuZIPWo_zeit_0_1"),
        509: Unknown("ID_Einst_SuZIPWo_zeit_1_0"),
        510: Unknown("ID_Einst_SuZIPWo_zeit_1_1"),
        511: Unknown("ID_Einst_SuZIPWo_zeit_2_0"),
        512: Unknown("ID_Einst_SuZIPWo_zeit_2_1"),
        513: Unknown("ID_Einst_SuZIPWo_zeit_3_0"),
        514: Unknown("ID_Einst_SuZIPWo_zeit_3_1"),
        515: Unknown("ID_Einst_SuZIPWo_zeit_4_0"),
        516: Unknown("ID_Einst_SuZIPWo_zeit_4_1"),
        517: Unknown("ID_Einst_SuZIP25_zeit_0_0"),
        518: Unknown("ID_Einst_SuZIP25_zeit_0_1"),
        519: Unknown("ID_Einst_SuZIP25_zeit_1_0"),
        520: Unknown("ID_Einst_SuZIP25_zeit_1_1"),
        521: Unknown("ID_Einst_SuZIP25_zeit_2_0"),
        522: Unknown("ID_Einst_SuZIP25_zeit_2_1"),
        523: Unknown("ID_Einst_SuZIP25_zeit_3_0"),
        524: Unknown("ID_Einst_SuZIP25_zeit_3_1"),
        525: Unknown("ID_Einst_SuZIP25_zeit_4_0"),
        526: Unknown("ID_Einst_SuZIP25_zeit_4_1"),
        527: Unknown("ID_Einst_SuZIP25_zeit_0_2"),
        528: Unknown("ID_Einst_SuZIP25_zeit_0_3"),
        529: Unknown("ID_Einst_SuZIP25_zeit_1_2"),
        530: Unknown("ID_Einst_SuZIP25_zeit_1_3"),
        531: Unknown("ID_Einst_SuZIP25_zeit_2_2"),
        532: Unknown("ID_Einst_SuZIP25_zeit_2_3"),
        533: Unknown("ID_Einst_SuZIP25_zeit_3_2"),
        534: Unknown("ID_Einst_SuZIP25_zeit_3_3"),
        535: Unknown("ID_Einst_SuZIP25_zeit_4_2"),
        536: Unknown("ID_Einst_SuZIP25_zeit_4_3"),
        537: Unknown("ID_Einst_SuZIPTg_zeit_0_0"),
        538: Unknown("ID_Einst_SuZIPTg_zeit_0_1"),
        539: Unknown("ID_Einst_SuZIPTg_zeit_1_0"),
        540: Unknown("ID_Einst_SuZIPTg_zeit_1_1"),
        541: Unknown("ID_Einst_SuZIPTg_zeit_2_0"),
        542: Unknown("ID_Einst_SuZIPTg_zeit_2_1"),
        543: Unknown("ID_Einst_SuZIPTg_zeit_3_0"),
        544: Unknown("ID_Einst_SuZIPTg_zeit_3_1"),
        545: Unknown("ID_Einst_SuZIPTg_zeit_4_0"),
        546: Unknown("ID_Einst_SuZIPTg_zeit_4_1"),
        547: Unknown("ID_Einst_SuZIPTg_zeit_0_2"),
        548: Unknown("ID_Einst_SuZIPTg_zeit_0_3"),
        549: Unknown("ID_Einst_SuZIPTg_zeit_1_2"),
        550: Unknown("ID_Einst_SuZIPTg_zeit_1_3"),
        551: Unknown("ID_Einst_SuZIPTg_zeit_2_2"),
        552: Unknown("ID_Einst_SuZIPTg_zeit_2_3"),
        553: Unknown("ID_Einst_SuZIPTg_zeit_3_2"),
        554: Unknown("ID_Einst_SuZIPTg_zeit_3_3"),
        555: Unknown("ID_Einst_SuZIPTg_zeit_4_2"),
        556: Unknown("ID_Einst_SuZIPTg_zeit_4_3"),
        557: Unknown("ID_Einst_SuZIPTg_zeit_0_4"),
        558: Unknown("ID_Einst_SuZIPTg_zeit_0_5"),
        559: Unknown("ID_Einst_SuZIPTg_zeit_1_4"),
        560: Unknown("ID_Einst_SuZIPTg_zeit_1_5"),
        561: Unknown("ID_Einst_SuZIPTg_zeit_2_4"),
        562: Unknown("ID_Einst_SuZIPTg_zeit_2_5"),
        563: Unknown("ID_Einst_SuZIPTg_zeit_3_4"),
        564: Unknown("ID_Einst_SuZIPTg_zeit_3_5"),
        565: Unknown("ID_Einst_SuZIPTg_zeit_4_4"),
        566: Unknown("ID_Einst_SuZIPTg_zeit_4_5"),
        567: Unknown("ID_Einst_SuZIPTg_zeit_0_6"),
        568: Unknown("ID_Einst_SuZIPTg_zeit_0_7"),
        569: Unknown("ID_Einst_SuZIPTg_zeit_1_6"),
        570: Unknown("ID_Einst_SuZIPTg_zeit_1_7"),
        571: Unknown("ID_Einst_SuZIPTg_zeit_2_6"),
        572: Unknown("ID_Einst_SuZIPTg_zeit_2_7"),
        573: Unknown("ID_Einst_SuZIPTg_zeit_3_6"),
        574: Unknown("ID_Einst_SuZIPTg_zeit_3_7"),
        575: Unknown("ID_Einst_SuZIPTg_zeit_4_6"),
        576: Unknown("ID_Einst_SuZIPTg_zeit_4_7"),
        577: Unknown("ID_Einst_SuZIPTg_zeit_0_8"),
        578: Unknown("ID_Einst_SuZIPTg_zeit_0_9"),
        579: Unknown("ID_Einst_SuZIPTg_zeit_1_8"),
        580: Unknown("ID_Einst_SuZIPTg_zeit_1_9"),
        581: Unknown("ID_Einst_SuZIPTg_zeit_2_8"),
        582: Unknown("ID_Einst_SuZIPTg_zeit_2_9"),
        583: Unknown("ID_Einst_SuZIPTg_zeit_3_8"),
        584: Unknown("ID_Einst_SuZIPTg_zeit_3_9"),
        585: Unknown("ID_Einst_SuZIPTg_zeit_4_8"),
        586: Unknown("ID_Einst_SuZIPTg_zeit_4_9"),
        587: Unknown("ID_Einst_SuZIPTg_zeit_0_10"),
        588: Unknown("ID_Einst_SuZIPTg_zeit_0_11"),
        589: Unknown("ID_Einst_SuZIPTg_zeit_1_10"),
        590: Unknown("ID_Einst_SuZIPTg_zeit_1_11"),
        591: Unknown("ID_Einst_SuZIPTg_zeit_2_10"),
        592: Unknown("ID_Einst_SuZIPTg_zeit_2_11"),
        593: Unknown("ID_Einst_SuZIPTg_zeit_3_10"),
        594: Unknown("ID_Einst_SuZIPTg_zeit_3_11"),
        595: Unknown("ID_Einst_SuZIPTg_zeit_4_10"),
        596: Unknown("ID_Einst_SuZIPTg_zeit_4_11"),
        597: Unknown("ID_Einst_SuZIPTg_zeit_0_12"),
        598: Unknown("ID_Einst_SuZIPTg_zeit_0_13"),
        599: Unknown("ID_Einst_SuZIPTg_zeit_1_12"),
        600: Unknown("ID_Einst_SuZIPTg_zeit_1_13"),
        601: Unknown("ID_Einst_SuZIPTg_zeit_2_12"),
        602: Unknown("ID_Einst_SuZIPTg_zeit_2_13"),
        603: Unknown("ID_Einst_SuZIPTg_zeit_3_12"),
        604: Unknown("ID_Einst_SuZIPTg_zeit_3_13"),
        605: Unknown("ID_Einst_SuZIPTg_zeit_4_12"),
        606: Unknown("ID_Einst_SuZIPTg_zeit_4_13"),
        607: Unknown("ID_Einst_SuSwb_akt"),
        608: Unknown("ID_Einst_SuSwbWo_zeit_0_0"),
        609: Unknown("ID_Einst_SuSwbWo_zeit_0_1"),
        610: Unknown("ID_Einst_SuSwbWo_zeit_1_0"),
        611: Unknown("ID_Einst_SuSwbWo_zeit_1_1"),
        612: Unknown("ID_Einst_SuSwbWo_zeit_2_0"),
        613: Unknown("ID_Einst_SuSwbWo_zeit_2_1"),
        614: Unknown("ID_Einst_SuSwb25_zeit_0_0"),
        615: Unknown("ID_Einst_SuSwb25_zeit_0_1"),
        616: Unknown("ID_Einst_SuSwb25_zeit_1_0"),
        617: Unknown("ID_Einst_SuSwb25_zeit_1_1"),
        618: Unknown("ID_Einst_SuSwb25_zeit_2_0"),
        619: Unknown("ID_Einst_SuSwb25_zeit_2_1"),
        620: Unknown("ID_Einst_SuSwb25_zeit_0_2"),
        621: Unknown("ID_Einst_SuSwb25_zeit_0_3"),
        622: Unknown("ID_Einst_SuSwb25_zeit_1_2"),
        623: Unknown("ID_Einst_SuSwb25_zeit_1_3"),
        624: Unknown("ID_Einst_SuSwb25_zeit_2_2"),
        625: Unknown("ID_Einst_SuSwb25_zeit_2_3"),
        626: Unknown("ID_Einst_SuSwbTg_zeit_0_0"),
        627: Unknown("ID_Einst_SuSwbTg_zeit_0_1"),
        628: Unknown("ID_Einst_SuSwbTg_zeit_1_0"),
        629: Unknown("ID_Einst_SuSwbTg_zeit_1_1"),
        630: Unknown("ID_Einst_SuSwbTg_zeit_2_0"),
        631: Unknown("ID_Einst_SuSwbTg_zeit_2_1"),
        632: Unknown("ID_Einst_SuSwbTg_zeit_0_2"),
        633: Unknown("ID_Einst_SuSwbTg_zeit_0_3"),
        634: Unknown("ID_Einst_SuSwbTg_zeit_1_2"),
        635: Unknown("ID_Einst_SuSwbTg_zeit_1_3"),
        636: Unknown("ID_Einst_SuSwbTg_zeit_2_2"),
        637: Unknown("ID_Einst_SuSwbTg_zeit_2_3"),
        638: Unknown("ID_Einst_SuSwbTg_zeit_0_4"),
        639: Unknown("ID_Einst_SuSwbTg_zeit_0_5"),
        640: Unknown("ID_Einst_SuSwbTg_zeit_1_4"),
        641: Unknown("ID_Einst_SuSwbTg_zeit_1_5"),
        642: Unknown("ID_Einst_SuSwbTg_zeit_2_4"),
        643: Unknown("ID_Einst_SuSwbTg_zeit_2_5"),
        644: Unknown("ID_Einst_SuSwbTg_zeit_0_6"),
        645: Unknown("ID_Einst_SuSwbTg_zeit_0_7"),
        646: Unknown("ID_Einst_SuSwbTg_zeit_1_6"),
        647: Unknown("ID_Einst_SuSwbTg_zeit_1_7"),
        648: Unknown("ID_Einst_SuSwbTg_zeit_2_6"),
        649: Unknown("ID_Einst_SuSwbTg_zeit_2_7"),
        650: Unknown("ID_Einst_SuSwbTg_zeit_0_8"),
        651: Unknown("ID_Einst_SuSwbTg_zeit_0_9"),
        652: Unknown("ID_Einst_SuSwbTg_zeit_1_8"),
        653: Unknown("ID_Einst_SuSwbTg_zeit_1_9"),
        654: Unknown("ID_Einst_SuSwbTg_zeit_2_8"),
        655: Unknown("ID_Einst_SuSwbTg_zeit_2_9"),
        656: Unknown("ID_Einst_SuSwbTg_zeit_0_10"),
        657: Unknown("ID_Einst_SuSwbTg_zeit_0_11"),
        658: Unknown("ID_Einst_SuSwbTg_zeit_1_10"),
        659: Unknown("ID_Einst_SuSwbTg_zeit_1_11"),
        660: Unknown("ID_Einst_SuSwbTg_zeit_2_10"),
        661: Unknown("ID_Einst_SuSwbTg_zeit_2_11"),
        662: Unknown("ID_Einst_SuSwbTg_zeit_0_12"),
        663: Unknown("ID_Einst_SuSwbTg_zeit_0_13"),
        664: Unknown("ID_Einst_SuSwbTg_zeit_1_12"),
        665: Unknown("ID_Einst_SuSwbTg_zeit_1_13"),
        666: Unknown("ID_Einst_SuSwbTg_zeit_2_12"),
        667: Unknown("ID_Einst_SuSwbTg_zeit_2_13"),
        668: Unknown("ID_Zaehler_BetrZeitWP"),
        669: Unknown("ID_Zaehler_BetrZeitVD1"),
        670: Unknown("ID_Zaehler_BetrZeitVD2"),
        671: Unknown("ID_Zaehler_BetrZeitZWE1"),
        672: Unknown("ID_Zaehler_BetrZeitZWE2"),
        673: Unknown("ID_Zaehler_BetrZeitZWE3"),
        674: Unknown("ID_Zaehler_BetrZeitImpVD1"),
        675: Unknown("ID_Zaehler_BetrZeitImpVD2"),
        676: Unknown("ID_Zaehler_BetrZeitEZMVD1"),
        677: Unknown("ID_Zaehler_BetrZeitEZMVD2"),
        678: Unknown("ID_Einst_Entl_Typ_0"),
        679: Unknown("ID_Einst_Entl_Typ_1"),
        680: Unknown("ID_Einst_Entl_Typ_2"),
        681: Unknown("ID_Einst_Entl_Typ_3"),
        682: Unknown("ID_Einst_Entl_Typ_4"),
        683: Unknown("ID_Einst_Entl_Typ_5"),
        684: Unknown("ID_Einst_Entl_Typ_6"),
        685: Unknown("ID_Einst_Entl_Typ_7"),
        686: Unknown("ID_Einst_Entl_Typ_8"),
        687: Unknown("ID_Einst_Entl_Typ_9"),
        688: Unknown("ID_Einst_Entl_Typ_10"),
        689: Unknown("ID_Einst_Entl_Typ_11"),
        690: Unknown("ID_Einst_Entl_Typ_12"),
        691: Unknown("ID_Einst_Vorl_max_MK1"),
        692: Unknown("ID_Einst_Vorl_max_MK2"),
        693: Unknown("ID_SU_FrkdMK1"),
        694: Unknown("ID_SU_FrkdMK2"),
        695: Unknown("ID_Ba_Hz_MK1_akt"),
        696: Unknown("ID_Ba_Hz_MK2_akt"),
        697: Unknown("ID_Einst_Zirk_Ein_akt"),
        698: Unknown("ID_Einst_Zirk_Aus_akt"),
        699: Unknown("ID_Einst_Heizgrenze"),
        700: Unknown("ID_Einst_Heizgrenze_Temp"),
        701: Unknown("ID_VariablenIBNgespeichert"),
        702: Unknown("ID_SchonIBNAssistant"),
        703: Unknown("ID_Heizgrenze_0"),
        704: Unknown("ID_Heizgrenze_1"),
        705: Unknown("ID_Heizgrenze_2"),
        706: Unknown("ID_Heizgrenze_3"),
        707: Unknown("ID_Heizgrenze_4"),
        708: Unknown("ID_Heizgrenze_5"),
        709: Unknown("ID_Heizgrenze_6"),
        710: Unknown("ID_Heizgrenze_7"),
        711: Unknown("ID_Heizgrenze_8"),
        712: Unknown("ID_Heizgrenze_9"),
        713: Unknown("ID_Heizgrenze_10"),
        714: Unknown("ID_Heizgrenze_11"),
        715: Unknown("ID_SchemenIBNgewahlt"),
        716: Unknown("ID_Switchoff_file_0_0"),
        717: Unknown("ID_Switchoff_file_1_0"),
        718: Unknown("ID_Switchoff_file_2_0"),
        719: Unknown("ID_Switchoff_file_3_0"),
        720: Unknown("ID_Switchoff_file_4_0"),
        721: Unknown("ID_Switchoff_file_0_1"),
        722: Unknown("ID_Switchoff_file_1_1"),
        723: Unknown("ID_Switchoff_file_2_1"),
        724: Unknown("ID_Switchoff_file_3_1"),
        725: Unknown("ID_Switchoff_file_4_1"),
        726: Unknown("ID_DauerDatenLoggerAktiv"),
        727: Unknown("ID_Laufvar_Heizgrenze"),
        728: Unknown("ID_Zaehler_BetrZeitHz"),
        729: Unknown("ID_Zaehler_BetrZeitBW"),
        730: Unknown("ID_Zaehler_BetrZeitKue"),
        731: Unknown("ID_SU_FstdHz"),
        732: Unknown("ID_SU_FstdBw"),
        733: Unknown("ID_SU_FstdSwb"),
        734: Unknown("ID_SU_FstdMK1"),
        735: Unknown("ID_SU_FstdMK2"),
        736: Unknown("ID_FerienAbsenkungHz"),
        737: Unknown("ID_FerienAbsenkungMK1"),
        738: Unknown("ID_FerienAbsenkungMK2"),
        739: Unknown("ID_FerienModusAktivHz"),
        740: Unknown("ID_FerienModusAktivBw"),
        741: Unknown("ID_FerienModusAktivSwb"),
        742: Unknown("ID_FerienModusAktivMk1"),
        743: Unknown("ID_FerienModusAktivMk2"),
        744: Unknown("ID_DisplayContrast_akt"),
        745: Unknown("ID_Ba_Hz_saved"),
        746: Unknown("ID_Ba_Bw_saved"),
        747: Unknown("ID_Ba_Sw_saved"),
        748: Unknown("ID_Ba_Hz_MK1_saved"),
        749: Unknown("ID_Ba_Hz_MK2_saved"),
        750: Unknown("ID_AdresseIP_akt"),
        751: Unknown("ID_SubNetMask_akt"),
        752: Unknown("ID_Add_Broadcast_akt"),
        753: Unknown("ID_Add_StdGateway_akt"),
        754: Unknown("ID_DHCPServerAktiv_akt"),
        755: Unknown("ID_WebserverPasswort_1_akt"),
        756: Unknown("ID_WebserverPasswort_2_akt"),
        757: Unknown("ID_WebserverPasswort_3_akt"),
        758: Unknown("ID_WebserverPasswort_4_akt"),
        759: Unknown("ID_WebserverPasswort_5_akt"),
        760: Unknown("ID_WebserverPasswort_6_akt"),
        761: Unknown("ID_WebServerWerteBekommen"),
        762: Unknown("ID_Einst_ParBetr_akt"),
        763: Unknown("ID_Einst_WpAnz_akt"),
        764: Unknown("ID_Einst_PhrTime_akt"),
        765: Unknown("ID_Einst_HysPar_akt"),
        766: Unknown("ID_IP_PB_Slave_0"),
        767: Unknown("ID_IP_PB_Slave_1"),
        768: Unknown("ID_IP_PB_Slave_2"),
        769: Unknown("ID_IP_PB_Slave_3"),
        770: Unknown("ID_IP_PB_Slave_4"),
        771: Unknown("ID_IP_PB_Slave_5"),
        772: Unknown("ID_Einst_BwHup_akt_backup"),
        773: Unknown("ID_Einst_SuMk3_akt"),
        774: Unknown("ID_Einst_HzMK3E_akt"),
        775: Unknown("ID_Einst_HzMK3ANH_akt"),
        776: Unknown("ID_Einst_HzMK3ABS_akt"),
        777: Unknown("ID_Einst_HzMK3Hgr_akt"),
        778: Unknown("ID_Einst_HzFtMK3Vl_akt"),
        779: MixedCircuitMode("ID_Ba_Hz_MK3_akt", True),
        780: Unknown("ID_Einst_MK3Typ_akt"),
        781: Unknown("ID_Einst_RTypMK3_akt"),
        782: Unknown("ID_Einst_MK3LzFaktor_akt"),
        783: Unknown("ID_Einst_MK3PerFaktor_akt"),
        784: Unknown("ID_FerienModusAktivMk3"),
        785: Unknown("ID_SU_FrkdMK3"),
        786: Unknown("ID_FerienAbsenkungMK3"),
        787: Unknown("ID_SU_FstdMK3"),
        788: Unknown("ID_Einst_SuMk3_akt2"),
        789: Unknown("ID_Einst_SuMk3Wo_zeit_0_0"),
        790: Unknown("ID_Einst_SuMk3Wo_zeit_0_1"),
        791: Unknown("ID_Einst_SuMk3Wo_zeit_1_0"),
        792: Unknown("ID_Einst_SuMk3Wo_zeit_1_1"),
        793: Unknown("ID_Einst_SuMk3Wo_zeit_2_0"),
        794: Unknown("ID_Einst_SuMk3Wo_zeit_2_1"),
        795: Unknown("ID_Einst_SuMk325_zeit_0_0"),
        796: Unknown("ID_Einst_SuMk325_zeit_0_1"),
        797: Unknown("ID_Einst_SuMk325_zeit_1_0"),
        798: Unknown("ID_Einst_SuMk325_zeit_1_1"),
        799: Unknown("ID_Einst_SuMk325_zeit_2_0"),
        800: Unknown("ID_Einst_SuMk325_zeit_2_1"),
        801: Unknown("ID_Einst_SuMk325_zeit_0_2"),
        802: Unknown("ID_Einst_SuMk325_zeit_0_3"),
        803: Unknown("ID_Einst_SuMk325_zeit_1_2"),
        804: Unknown("ID_Einst_SuMk325_zeit_1_3"),
        805: Unknown("ID_Einst_SuMk325_zeit_2_2"),
        806: Unknown("ID_Einst_SuMk325_zeit_2_3"),
        807: Unknown("ID_Einst_SuMk3Tg_zeit_0_0"),
        808: Unknown("ID_Einst_SuMk3Tg_zeit_0_1"),
        809: Unknown("ID_Einst_SuMk3Tg_zeit_1_0"),
        810: Unknown("ID_Einst_SuMk3Tg_zeit_1_1"),
        811: Unknown("ID_Einst_SuMk3Tg_zeit_2_0"),
        812: Unknown("ID_Einst_SuMk3Tg_zeit_2_1"),
        813: Unknown("ID_Einst_SuMk3Tg_zeit_0_2"),
        814: Unknown("ID_Einst_SuMk3Tg_zeit_0_3"),
        815: Unknown("ID_Einst_SuMk3Tg_zeit_1_2"),
        816: Unknown("ID_Einst_SuMk3Tg_zeit_1_3"),
        817: Unknown("ID_Einst_SuMk3Tg_zeit_2_2"),
        818: Unknown("ID_Einst_SuMk3Tg_zeit_2_3"),
        819: Unknown("ID_Einst_SuMk3Tg_zeit_0_4"),
        820: Unknown("ID_Einst_SuMk3Tg_zeit_0_5"),
        821: Unknown("ID_Einst_SuMk3Tg_zeit_1_4"),
        822: Unknown("ID_Einst_SuMk3Tg_zeit_1_5"),
        823: Unknown("ID_Einst_SuMk3Tg_zeit_2_4"),
        824: Unknown("ID_Einst_SuMk3Tg_zeit_2_5"),
        825: Unknown("ID_Einst_SuMk3Tg_zeit_0_6"),
        826: Unknown("ID_Einst_SuMk3Tg_zeit_0_7"),
        827: Unknown("ID_Einst_SuMk3Tg_zeit_1_6"),
        828: Unknown("ID_Einst_SuMk3Tg_zeit_1_7"),
        829: Unknown("ID_Einst_SuMk3Tg_zeit_2_6"),
        830: Unknown("ID_Einst_SuMk3Tg_zeit_2_7"),
        831: Unknown("ID_Einst_SuMk3Tg_zeit_0_8"),
        832: Unknown("ID_Einst_SuMk3Tg_zeit_0_9"),
        833: Unknown("ID_Einst_SuMk3Tg_zeit_1_8"),
        834: Unknown("ID_Einst_SuMk3Tg_zeit_1_9"),
        835: Unknown("ID_Einst_SuMk3Tg_zeit_2_8"),
        836: Unknown("ID_Einst_SuMk3Tg_zeit_2_9"),
        837: Unknown("ID_Einst_SuMk3Tg_zeit_0_10"),
        838: Unknown("ID_Einst_SuMk3Tg_zeit_0_11"),
        839: Unknown("ID_Einst_SuMk3Tg_zeit_1_10"),
        840: Unknown("ID_Einst_SuMk3Tg_zeit_1_11"),
        841: Unknown("ID_Einst_SuMk3Tg_zeit_2_10"),
        842: Unknown("ID_Einst_SuMk3Tg_zeit_2_11"),
        843: Unknown("ID_Einst_SuMk3Tg_zeit_0_12"),
        844: Unknown("ID_Einst_SuMk3Tg_zeit_0_13"),
        845: Unknown("ID_Einst_SuMk3Tg_zeit_1_12"),
        846: Unknown("ID_Einst_SuMk3Tg_zeit_1_13"),
        847: Unknown("ID_Einst_SuMk3Tg_zeit_2_12"),
        848: Unknown("ID_Einst_SuMk3Tg_zeit_2_13"),
        849: Unknown("ID_Ba_Hz_MK3_saved"),
        850: Hours("ID_Einst_Kuhl_Zeit_Ein_akt", True),
        851: Hours("ID_Einst_Kuhl_Zeit_Aus_akt", True),
        852: Unknown("ID_Waermemenge_Seit"),
        853: Unknown("ID_Waermemenge_WQ"),
        854: Unknown("ID_Waermemenge_Hz"),
        855: Unknown("ID_Waermemenge_WQ_ges"),
        856: Unknown("ID_Einst_Entl_Typ_13"),
        857: Unknown("ID_Einst_Entl_Typ_14"),
        858: Unknown("ID_Einst_Entl_Typ_15"),
        859: Unknown("ID_Zaehler_BetrZeitSW"),
        860: Unknown("ID_Einst_Fernwartung_akt"),
        861: Unknown("ID_AdresseIPServ_akt"),
        862: Unknown("ID_Einst_TA_EG_akt"),
        863: Unknown("ID_Einst_TVLmax_EG_akt"),
        864: Unknown("ID_Einst_Popt_Nachlauf_akt"),
        865: Unknown("ID_FernwartungVertrag_akt"),
        866: Unknown("ID_FernwartungAktuZeit"),
        867: Unknown("ID_Einst_Effizienzpumpe_Nominal_akt"),
        868: Unknown("ID_Einst_Effizienzpumpe_Minimal_akt"),
        869: Unknown("ID_Einst_Effizienzpumpe_akt"),
        870: Unknown("ID_Einst_Waermemenge_akt"),
        871: Unknown("ID_Einst_Wm_Versorgung_Korrektur_akt"),
        872: Unknown("ID_Einst_Wm_Auswertung_Korrektur_akt"),
        873: Unknown("ID_SoftwareUpdateJetztGemacht_akt"),
        874: Unknown("ID_WP_SerienNummer_DATUM"),
        875: Unknown("ID_WP_SerienNummer_HEX"),
        876: Unknown("ID_WP_SerienNummer_INDEX"),
        877: Unknown("ID_ProgWerteWebSrvBeobarten"),
        878: Unknown("ID_Waermemenge_BW"),
        879: Unknown("ID_Waermemenge_SW"),
        880: Unknown("ID_Waermemenge_Datum"),
        881: SolarMode("ID_Einst_Solar_akt", True),
        882: Unknown("ID_BSTD_Solar"),
        883: Unknown("ID_Einst_TDC_Koll_Max_akt"),
        884: Unknown("ID_Einst_Akt_Kuehlung_akt"),
        885: Unknown("ID_Einst_Vorlauf_VBO_akt"),
        886: Unknown("ID_Einst_KRHyst_akt"),
        887: Unknown("ID_Einst_Akt_Kuehl_Speicher_min_akt"),
        888: Unknown("ID_Einst_Akt_Kuehl_Freig_WQE_akt"),
        889: Unknown("ID_NDAB_WW_Anzahl"),
        890: Unknown("ID_NDS_WW_KD_Quitt"),
        891: Unknown("ID_Einst_AbtZykMin_akt"),
        892: Unknown("ID_Einst_VD2_Zeit_Min_akt"),
        893: Unknown("ID_Einst_Hysterese_HR_verkuerzt_akt"),
        894: VentilationMode("ID_Einst_BA_Lueftung_akt", True),
        895: Unknown("ID_Einst_SuLuf_akt"),
        896: Unknown("ID_Einst_SuLufWo_zeit_0_0_0"),
        897: Unknown("ID_Einst_SuLufWo_zeit_0_1_0"),
        898: Unknown("ID_Einst_SuLufWo_zeit_0_2_0"),
        899: Unknown("ID_Einst_SuLuf25_zeit_0_0_0"),
        900: Unknown("ID_Einst_SuLuf25_zeit_0_1_0"),
        901: Unknown("ID_Einst_SuLuf25_zeit_0_2_0"),
        902: Unknown("ID_Einst_SuLuf25_zeit_0_0_2"),
        903: Unknown("ID_Einst_SuLuf25_zeit_0_1_2"),
        904: Unknown("ID_Einst_SuLuf25_zeit_0_2_2"),
        905: Unknown("ID_Einst_SuLufTg_zeit_0_0_0"),
        906: Unknown("ID_Einst_SuLufTg_zeit_0_1_0"),
        907: Unknown("ID_Einst_SuLufTg_zeit_0_2_0"),
        908: Unknown("ID_Einst_SuLufTg_zeit_0_0_2"),
        909: Unknown("ID_Einst_SuLufTg_zeit_0_1_2"),
        910: Unknown("ID_Einst_SuLufTg_zeit_0_2_2"),
        911: Unknown("ID_Einst_SuLufTg_zeit_0_0_4"),
        912: Unknown("ID_Einst_SuLufTg_zeit_0_1_4"),
        913: Unknown("ID_Einst_SuLufTg_zeit_0_2_4"),
        914: Unknown("ID_Einst_SuLufTg_zeit_0_0_6"),
        915: Unknown("ID_Einst_SuLufTg_zeit_0_1_6"),
        916: Unknown("ID_Einst_SuLufTg_zeit_0_2_6"),
        917: Unknown("ID_Einst_SuLufTg_zeit_0_0_8"),
        918: Unknown("ID_Einst_SuLufTg_zeit_0_1_8"),
        919: Unknown("ID_Einst_SuLufTg_zeit_0_2_8"),
        920: Unknown("ID_Einst_SuLufTg_zeit_0_0_10"),
        921: Unknown("ID_Einst_SuLufTg_zeit_0_1_10"),
        922: Unknown("ID_Einst_SuLufTg_zeit_0_2_10"),
        923: Unknown("ID_Einst_SuLufTg_zeit_0_0_12"),
        924: Unknown("ID_Einst_SuLufTg_zeit_0_1_12"),
        925: Unknown("ID_Einst_SuLufTg_zeit_0_2_12"),
        926: Unknown("ID_Einst_SuLufWo_zeit_1_0_0"),
        927: Unknown("ID_Einst_SuLufWo_zeit_1_1_0"),
        928: Unknown("ID_Einst_SuLufWo_zeit_1_2_0"),
        929: Unknown("ID_Einst_SuLuf25_zeit_1_0_0"),
        930: Unknown("ID_Einst_SuLuf25_zeit_1_1_0"),
        931: Unknown("ID_Einst_SuLuf25_zeit_1_2_0"),
        932: Unknown("ID_Einst_SuLuf25_zeit_1_0_2"),
        933: Unknown("ID_Einst_SuLuf25_zeit_1_1_2"),
        934: Unknown("ID_Einst_SuLuf25_zeit_1_2_2"),
        935: Unknown("ID_Einst_SuLufTg_zeit_1_0_0"),
        936: Unknown("ID_Einst_SuLufTg_zeit_1_1_0"),
        937: Unknown("ID_Einst_SuLufTg_zeit_1_2_0"),
        938: Unknown("ID_Einst_SuLufTg_zeit_1_0_2"),
        939: Unknown("ID_Einst_SuLufTg_zeit_1_1_2"),
        940: Unknown("ID_Einst_SuLufTg_zeit_1_2_2"),
        941: Unknown("ID_Einst_SuLufTg_zeit_1_0_4"),
        942: Unknown("ID_Einst_SuLufTg_zeit_1_1_4"),
        943: Unknown("ID_Einst_SuLufTg_zeit_1_2_4"),
        944: Unknown("ID_Einst_SuLufTg_zeit_1_0_6"),
        945: Unknown("ID_Einst_SuLufTg_zeit_1_1_6"),
        946: Unknown("ID_Einst_SuLufTg_zeit_1_2_6"),
        947: Unknown("ID_Einst_SuLufTg_zeit_1_0_8"),
        948: Unknown("ID_Einst_SuLufTg_zeit_1_1_8"),
        949: Unknown("ID_Einst_SuLufTg_zeit_1_2_8"),
        950: Unknown("ID_Einst_SuLufTg_zeit_1_0_10"),
        951: Unknown("ID_Einst_SuLufTg_zeit_1_1_10"),
        952: Unknown("ID_Einst_SuLufTg_zeit_1_2_10"),
        953: Unknown("ID_Einst_SuLufTg_zeit_1_0_12"),
        954: Unknown("ID_Einst_SuLufTg_zeit_1_1_12"),
        955: Unknown("ID_Einst_SuLufTg_zeit_1_2_12"),
        956: Unknown("ID_FerienModusAktivLueftung"),
        957: Unknown("ID_Einst_BA_Lueftung_saved"),
        958: Unknown("ID_SU_FrkdLueftung"),
        959: Unknown("ID_SU_FstdLueftung"),
        960: Unknown("ID_Einst_Luf_Feuchteschutz_akt"),
        961: Unknown("ID_Einst_Luf_Reduziert_akt"),
        962: Unknown("ID_Einst_Luf_Nennlueftung_akt"),
        963: Unknown("ID_Einst_Luf_Intensivlueftung_akt"),
        964: Unknown("ID_Timer_Fil_4Makt"),
        965: Unknown("ID_Timer_Fil_WoAkt"),
        966: Celsius("ID_Sollwert_KuCft3_akt", True),
        967: Celsius("ID_Sollwert_AtDif3_akt", True),
        968: Unknown("ID_Bitmaske_0"),
        969: Unknown("ID_Einst_Lueftungsstufen"),
        970: Unknown("ID_SysEin_Meldung_TDI"),
        971: Unknown("ID_SysEin_Typ_WZW"),
        972: Unknown("ID_Einst_GLT_aktiviert"),
        973: Unknown("ID_Einst_BW_max"),
        974: Unknown("ID_Einst_Sollwert_TRL_Kuehlen"),
        975: Unknown("ID_Einst_Medium_Waermequelle"),
        976: Unknown("ID_Einst_Photovoltaik_akt"),
        977: Unknown("ID_Einst_Multispeicher_akt"),
        978: Unknown("ID_Einst_PKuehlTime_akt"),
        979: Unknown("ID_Einst_Minimale_Ruecklaufsolltemperatur"),
        980: Unknown("ID_RBE_Einflussfaktor_RT_akt"),
        981: Unknown("ID_RBE_Freigabe_Kuehlung_akt"),
        982: Unknown("ID_RBE_Waermeverteilsystem_akt"),
        983: Unknown("ID_RBE_Zeit_Heizstab_aktiv"),
        984: Unknown("ID_SEC_ND_Alarmgrenze"),
        985: Unknown("ID_SEC_HD_Alarmgrenze"),
        986: Unknown("ID_SEC_Abtauendtemperatur"),
        987: Unknown("ID_Einst_Min_RPM_BW"),
        988: Unknown("ID_Einst_Luf_Feuchteschutz_Faktor_akt"),
        989: Unknown("ID_Einst_Luf_Reduziert_Faktor_akt"),
        990: Unknown("ID_Einst_Luf_Nennlueftung_Faktor_akt"),
        991: Unknown("ID_Einst_Luf_Intensivlueftung_Faktor_akt"),
        992: Unknown("ID_Einst_Freigabe_Zeit_ZWE"),
        993: Unknown("ID_Einst_min_VL_Kuehl"),
        994: Unknown("ID_Einst_Warmwasser_Nachheizung"),
        995: Unknown("ID_Switchoff_file_LWD2_0_0"),
        996: Unknown("ID_Switchoff_file_LWD2_1_0"),
        997: Unknown("ID_Switchoff_file_LWD2_2_0"),
        998: Unknown("ID_Switchoff_file_LWD2_3_0"),
        999: Unknown("ID_Switchoff_file_LWD2_4_0"),
        1000: Unknown("ID_Switchoff_file_LWD2_0_1"),
        1001: Unknown("ID_Switchoff_file_LWD2_1_1"),
        1002: Unknown("ID_Switchoff_file_LWD2_2_1"),
        1003: Unknown("ID_Switchoff_file_LWD2_3_1"),
        1004: Unknown("ID_Switchoff_file_LWD2_4_1"),
        1005: Unknown("ID_Switchoff_index_LWD2"),
        1006: Unknown("ID_Einst_Effizienzpumpe_Nominal_2"),
        1007: Unknown("ID_Einst_Effizienzpumpe_Minimal_2"),
        1008: Unknown("ID_Einst_Wm_Versorgung_Korrektur_2"),
        1009: Unknown("ID_Einst_Wm_Auswertung_Korrektur_2"),
        1010: Unknown("ID_Einst_isTwin"),
        1011: Unknown("ID_Einst_TAmin_2"),
        1012: Unknown("ID_Einst_TVLmax_2"),
        1013: Unknown("ID_Einst_TA_EG_2"),
        1014: Unknown("ID_Einst_TVLmax_EG_2"),
        1015: Unknown("ID_Waermemenge_Hz_2"),
        1016: Unknown("ID_Waermemenge_BW_2"),
        1017: Unknown("ID_Waermemenge_SW_2"),
        1018: Unknown("ID_Waermemenge_Seit_2"),
        1019: Unknown("ID_Einst_Entl_Typ_15_2"),
        1020: Unknown("ID_Einst_WW_Nachheizung_max"),
        1021: Unknown("ID_Einst_Kuhl_Zeit_Ein_RT"),
        1022: Unknown("ID_Einst_ZWE1_Pos"),
        1023: Unknown("ID_Einst_ZWE2_Pos"),
        1024: Unknown("ID_Einst_ZWE3_Pos"),
        1025: Unknown("ID_Einst_Leistung_ZWE"),
        1026: Unknown("ID_WP_SN2_DATUM"),
        1027: Unknown("ID_WP_SN2_HEX"),
        1028: Unknown("ID_WP_SN2_INDEX"),
        1029: Unknown("ID_CWP_saved2"),
        1030: Unknown("ID_Einst_SmartGrid"),
        1031: Unknown("ID_Einst_P155_HDS"),
        1032: Unknown("ID_Einst_P155_PumpHeat_Max"),
        1033: Unknown("ID_Einst_P155_PumpHeatCtrl"),
        1034: Unknown("ID_Einst_P155_PumpDHWCtrl"),
        1035: Unknown("ID_Einst_P155_PumpDHW_RPM"),
        1036: Unknown("ID_Einst_P155_PumpPoolCtrl"),
        1037: Unknown("ID_Einst_P155_PumpPool_RPM"),
        1038: Unknown("ID_Einst_P155_PumpCool_RPM"),
        1039: Unknown("ID_Einst_P155_PumpVBOCtrl"),
        1040: Unknown("ID_Einst_P155_PumpVBO_RPM_C"),
        1041: Unknown("ID_Einst_P155_PumpDHW_Max"),
        1042: Unknown("ID_Einst_P155_PumpPool_Max"),
        1043: Unknown("ID_Einst_P155_Sperrband_1"),
        1044: Unknown("ID_Einst_P155_Leistungsfreigabe"),
        1045: Unknown("ID_Einst_P155_DHW_Freq"),
        1046: Unknown("ID_Einst_SWHUP"),
        1047: Unknown("ID_Einst_P155_SWB_Freq"),
        1048: Unknown("ID_Einst_MK1_Regelung"),
        1049: Unknown("ID_Einst_MK2_Regelung"),
        1050: Unknown("ID_Einst_MK3_Regelung"),
        1051: Unknown("ID_Einst_PV_WW_Sperrzeit"),
        1052: Unknown("ID_Einst_Warmwasser_extra"),
        1053: Unknown("ID_Einst_Vorl_akt_Kuehl"),
        1054: Unknown("ID_WP_SN3_DATUM"),
        1055: Unknown("ID_WP_SN3_HEX"),
        1056: Unknown("ID_WP_SN3_INDEX"),
        1057: Unknown("ID_Einst_Vorlauf_ZUP"),
        1058: Unknown("ID_Einst_Abtauen_im_Warmwasser"),
        1059: Unknown("ID_Waermemenge_ZWE"),
        1060: Unknown("ID_Waermemenge_Reset"),
        1061: Unknown("ID_Waermemenge_Reset_2"),
        1062: Unknown("ID_Einst_Brunnenpumpe_min"),
        1063: Unknown("ID_Einst_Brunnenpumpe_max"),
        1064: Unknown("ID_Einst_SmartHomeID"),
        1065: Unknown("ID_Einst_SmartHK"),
        1066: Unknown("ID_Einst_SmartMK1"),
        1067: Unknown("ID_Einst_SmartMK2"),
        1068: Unknown("ID_Einst_SmartMK3"),
        1069: Unknown("ID_Einst_SmartWW"),
        1070: Unknown("ID_Einst_SmartDefrost"),
        1071: Unknown("ID_Einst_Empty1071"),
        1072: Unknown("ID_Einst_MinVLMK1"),
        1073: Unknown("ID_Einst_MinVLMK2"),
        1074: Unknown("ID_Einst_MinVLMK3"),
        1075: Unknown("ID_Einst_MaxVLMK1"),
        1076: Unknown("ID_Einst_MaxVLMK2"),
        1077: Unknown("ID_Einst_MaxVLMK3"),
        1078: Unknown("ID_Einst_SmartPlusHz"),
        1079: Unknown("ID_Einst_SmartMinusHz"),
        1080: Unknown("ID_Einst_SmartPlusMK1"),
        1081: Unknown("ID_Einst_SmartMinusMK1"),
        1082: Unknown("ID_Einst_SmartPlusMK2"),
        1083: Unknown("ID_Einst_SmartMinusMK2"),
        1084: Unknown("ID_Einst_SmartPlusMK3"),
        1085: Unknown("ID_Einst_SmartMinusMK3"),
        1085: Unknown("Unknown_Parameter_1085"),
        1086: Unknown("Unknown_Parameter_1086"),
        1087: Unknown("Unknown_Parameter_1087"),
        1088: Unknown("Unknown_Parameter_1088"),
        1089: Unknown("Unknown_Parameter_1089"),
        1090: Unknown("Unknown_Parameter_1090"),
        1091: Unknown("Unknown_Parameter_1091"),
        1092: Unknown("Unknown_Parameter_1092"),
        1093: Unknown("Unknown_Parameter_1093"),
        1094: Unknown("Unknown_Parameter_1094"),
        1095: Unknown("Unknown_Parameter_1095"),
        1096: Unknown("Unknown_Parameter_1096"),
        1097: Unknown("Unknown_Parameter_1097"),
        1098: Unknown("Unknown_Parameter_1098"),
        1099: Unknown("Unknown_Parameter_1099"),
        1100: Unknown("Unknown_Parameter_1100"),
        1101: Unknown("Unknown_Parameter_1101"),
        1102: Unknown("Unknown_Parameter_1102"),
        1103: Unknown("Unknown_Parameter_1103"),
        1104: Unknown("Unknown_Parameter_1104"),
        1105: Unknown("Unknown_Parameter_1105"),
        1106: Unknown("Unknown_Parameter_1106"),
        1107: Unknown("Unknown_Parameter_1107"),
        1108: Unknown("Unknown_Parameter_1108"),
        1109: Unknown("Unknown_Parameter_1109"),
        1110: Unknown("Unknown_Parameter_1110"),
        1111: Unknown("Unknown_Parameter_1111"),
        1112: Unknown("Unknown_Parameter_1112"),
        1113: Unknown("Unknown_Parameter_1113"),
        1114: Unknown("Unknown_Parameter_1114"),
        1115: Unknown("Unknown_Parameter_1115"),
        1116: Unknown("Unknown_Parameter_1116"),
        1117: Unknown("Unknown_Parameter_1117"),
        1118: Unknown("Unknown_Parameter_1118"),
        1119: Unknown("Unknown_Parameter_1119"),
    }

    def __init__(self, safe=True):
        """Initialize parameters class."""
        self.safe = safe
        self._queue = {}

    def _parse(self, data):
        """Parse raw parameter data."""
        for i, d in enumerate(data):
            p = self.parameters.get(i, False)
            if p is not False:
                p.value = p._to(d)
            else:
                LOGGER.warn(f"Parameter '{i}' not in list of parameters")

    def _lookup(self, p):
        """Lookup parameter by either id or name."""
        if isinstance(p, int):
            return p, self.parameters.get(p, None)
        if isinstance(p, str):
            try:
                p = int(p)
                return p, self.parameters.get(p, None)
            except ValueError:
                for k, v in self.parameters.items():
                    if v.name == p:
                        return k, v
        LOGGER.warn(f"Parameter '{p}' not found")
        return None, None

    def get(self, p):
        """Get parameter by id or name."""
        id, parameter = self._lookup(p)
        return parameter

    def set(self, p, v):
        """Set parameter to new value."""
        id, parameter = self._lookup(p)
        if id:
            if parameter.writeable or not self.safe:
                self._queue[id] = parameter._from(v)
            else:
                LOGGER.warn(f"Parameter {parameter.name} not safe for writing!")



import logging

from luxtronik.datatypes import *

logging.basicConfig(level="WARNING")
LOGGER = logging.getLogger("Luxtronik.Calculations")


class Calculations:
    """Class that holds all calculations."""

    calculations = {
        0: Unknown("Unknown_Calculation_0"),
        1: Unknown("Unknown_Calculation_1"),
        2: Unknown("Unknown_Calculation_2"),
        3: Unknown("Unknown_Calculation_3"),
        4: Unknown("Unknown_Calculation_4"),
        5: Unknown("Unknown_Calculation_5"),
        6: Unknown("Unknown_Calculation_6"),
        7: Unknown("Unknown_Calculation_7"),
        8: Unknown("Unknown_Calculation_8"),
        9: Unknown("Unknown_Calculation_9"),
        10: Celsius("ID_WEB_Temperatur_TVL"),
        11: Celsius("ID_WEB_Temperatur_TRL"),
        12: Celsius("ID_WEB_Sollwert_TRL_HZ"),
        13: Celsius("ID_WEB_Temperatur_TRL_ext"),
        14: Celsius("ID_WEB_Temperatur_THG"),
        15: Celsius("ID_WEB_Temperatur_TA"),
        16: Celsius("ID_WEB_Mitteltemperatur"),
        17: Celsius("ID_WEB_Temperatur_TBW"),
        18: Celsius("ID_WEB_Einst_BWS_akt"),
        19: Celsius("ID_WEB_Temperatur_TWE"),
        20: Celsius("ID_WEB_Temperatur_TWA"),
        21: Celsius("ID_WEB_Temperatur_TFB1"),
        22: Celsius("ID_WEB_Sollwert_TVL_MK1"),
        23: Celsius("ID_WEB_Temperatur_RFV"),
        24: Celsius("ID_WEB_Temperatur_TFB2"),
        25: Celsius("ID_WEB_Sollwert_TVL_MK2"),
        26: Celsius("ID_WEB_Temperatur_TSK"),
        27: Celsius("ID_WEB_Temperatur_TSS"),
        28: Celsius("ID_WEB_Temperatur_TEE"),
        29: Bool("ID_WEB_ASDin"),
        30: Bool("ID_WEB_BWTin"),
        31: Bool("ID_WEB_EVUin"),
        32: Bool("ID_WEB_HDin"),
        33: Bool("ID_WEB_MOTin"),
        34: Bool("ID_WEB_NDin"),
        35: Bool("ID_WEB_PEXin"),
        36: Bool("ID_WEB_SWTin"),
        37: Bool("ID_WEB_AVout"),
        38: Bool("ID_WEB_BUPout"),
        39: Bool("ID_WEB_HUPout"),
        40: Bool("ID_WEB_MA1out"),
        41: Bool("ID_WEB_MZ1out"),
        42: Bool("ID_WEB_VENout"),
        43: Bool("ID_WEB_VBOout"),
        44: Bool("ID_WEB_VD1out"),
        45: Bool("ID_WEB_VD2out"),
        46: Bool("ID_WEB_ZIPout"),
        47: Bool("ID_WEB_ZUPout"),
        48: Bool("ID_WEB_ZW1out"),
        49: Bool("ID_WEB_ZW2SSTout"),
        50: Bool("ID_WEB_ZW3SSTout"),
        51: Bool("ID_WEB_FP2out"),
        52: Bool("ID_WEB_SLPout"),
        53: Bool("ID_WEB_SUPout"),
        54: Bool("ID_WEB_MZ2out"),
        55: Bool("ID_WEB_MA2out"),
        56: Seconds("ID_WEB_Zaehler_BetrZeitVD1"),
        57: Pulses("ID_WEB_Zaehler_BetrZeitImpVD1"),
        58: Seconds("ID_WEB_Zaehler_BetrZeitVD2"),
        59: Pulses("ID_WEB_Zaehler_BetrZeitImpVD2"),
        60: Seconds("ID_WEB_Zaehler_BetrZeitZWE1"),
        61: Seconds("ID_WEB_Zaehler_BetrZeitZWE2"),
        62: Seconds("ID_WEB_Zaehler_BetrZeitZWE3"),
        63: Seconds("ID_WEB_Zaehler_BetrZeitWP"),
        64: Seconds("ID_WEB_Zaehler_BetrZeitHz"),
        65: Seconds("ID_WEB_Zaehler_BetrZeitBW"),
        66: Seconds("ID_WEB_Zaehler_BetrZeitKue"),
        67: Seconds("ID_WEB_Time_WPein_akt"),
        68: Seconds("ID_WEB_Time_ZWE1_akt"),
        69: Seconds("ID_WEB_Time_ZWE2_akt"),
        70: Seconds("ID_WEB_Timer_EinschVerz"),
        71: Seconds("ID_WEB_Time_SSPAUS_akt"),
        72: Seconds("ID_WEB_Time_SSPEIN_akt"),
        73: Seconds("ID_WEB_Time_VDStd_akt"),
        74: Seconds("ID_WEB_Time_HRM_akt"),
        75: Seconds("ID_WEB_Time_HRW_akt"),
        76: Seconds("ID_WEB_Time_LGS_akt"),
        77: Seconds("ID_WEB_Time_SBW_akt"),
        78: Code_WP("ID_WEB_Code_WP_akt"),
        79: BivalenceLevel("ID_WEB_BIV_Stufe_akt"),
        80: OperationMode("ID_WEB_WP_BZ_akt"),
        81: Version("ID_WEB_SoftStand"),
        91: IPAddress("ID_WEB_AdresseIP_akt"),
        92: IPAddress("ID_WEB_SubNetMask_akt"),
        93: IPAddress("ID_WEB_Add_Broadcast"),
        94: IPAddress("ID_WEB_Add_StdGateway"),
        95: Timestamp("ID_WEB_ERROR_Time0"),
        96: Timestamp("ID_WEB_ERROR_Time1"),
        97: Timestamp("ID_WEB_ERROR_Time2"),
        98: Timestamp("ID_WEB_ERROR_Time3"),
        99: Timestamp("ID_WEB_ERROR_Time4"),
        100: Errorcode("ID_WEB_ERROR_Nr0"),
        101: Errorcode("ID_WEB_ERROR_Nr1"),
        102: Errorcode("ID_WEB_ERROR_Nr2"),
        103: Errorcode("ID_WEB_ERROR_Nr3"),
        104: Errorcode("ID_WEB_ERROR_Nr4"),
        105: Count("ID_WEB_AnzahlFehlerInSpeicher"),
        106: SwitchoffFile("ID_WEB_Switchoff_file_Nr0"),
        107: SwitchoffFile("ID_WEB_Switchoff_file_Nr1"),
        108: SwitchoffFile("ID_WEB_Switchoff_file_Nr2"),
        109: SwitchoffFile("ID_WEB_Switchoff_file_Nr3"),
        110: SwitchoffFile("ID_WEB_Switchoff_file_Nr4"),
        111: Timestamp("ID_WEB_Switchoff_file_Time0"),
        112: Timestamp("ID_WEB_Switchoff_file_Time1"),
        113: Timestamp("ID_WEB_Switchoff_file_Time2"),
        114: Timestamp("ID_WEB_Switchoff_file_Time3"),
        115: Timestamp("ID_WEB_Switchoff_file_Time4"),
        116: Bool("ID_WEB_Comfort_exists"),
        117: MainMenuStatusLine1("ID_WEB_HauptMenuStatus_Zeile1"),
        118: MainMenuStatusLine2("ID_WEB_HauptMenuStatus_Zeile2"),
        119: MainMenuStatusLine3("ID_WEB_HauptMenuStatus_Zeile3"),
        120: Seconds("ID_WEB_HauptMenuStatus_Zeit"),
        121: Level("ID_WEB_HauptMenuAHP_Stufe"),
        122: Celsius("ID_WEB_HauptMenuAHP_Temp"),
        123: Seconds("ID_WEB_HauptMenuAHP_Zeit"),
        124: Bool("ID_WEB_SH_BWW"),
        125: Icon("ID_WEB_SH_HZ"),
        126: Icon("ID_WEB_SH_MK1"),
        127: Icon("ID_WEB_SH_MK2"),
        128: Unknown("ID_WEB_Einst_Kurzrpgramm"),
        129: Unknown("ID_WEB_StatusSlave_1"),
        130: Unknown("ID_WEB_StatusSlave_2"),
        131: Unknown("ID_WEB_StatusSlave_3"),
        132: Unknown("ID_WEB_StatusSlave_4"),
        133: Unknown("ID_WEB_StatusSlave_5"),
        134: Timestamp("ID_WEB_AktuelleTimeStamp"),
        135: Icon("ID_WEB_SH_MK3"),
        136: Celsius("ID_WEB_Sollwert_TVL_MK3"),
        137: Celsius("ID_WEB_Temperatur_TFB3"),
        138: Bool("ID_WEB_MZ3out"),
        139: Bool("ID_WEB_MA3out"),
        140: Bool("ID_WEB_FP3out"),
        141: Seconds("ID_WEB_Time_AbtIn"),
        142: Celsius("ID_WEB_Temperatur_RFV2"),
        143: Celsius("ID_WEB_Temperatur_RFV3"),
        144: Icon("ID_WEB_SH_SW"),
        145: Unknown("ID_WEB_Zaehler_BetrZeitSW"),
        146: Bool("ID_WEB_FreigabKuehl"),
        147: Voltage("ID_WEB_AnalogIn"),
        148: Unknown("ID_WEB_SonderZeichen"),
        149: Icon("ID_WEB_SH_ZIP"),
        150: Icon("ID_WEB_WebsrvProgrammWerteBeobarten"),
        151: Energy("ID_WEB_WMZ_Heizung"),
        152: Energy("ID_WEB_WMZ_Brauchwasser"),
        153: Energy("ID_WEB_WMZ_Schwimmbad"),
        154: Energy("ID_WEB_WMZ_Seit"),
        155: Flow("ID_WEB_WMZ_Durchfluss"),
        156: Voltage("ID_WEB_AnalogOut1"),
        157: Voltage("ID_WEB_AnalogOut2"),
        158: Seconds("ID_WEB_Time_Heissgas"),
        159: Celsius("ID_WEB_Temp_Lueftung_Zuluft"),
        160: Celsius("ID_WEB_Temp_Lueftung_Abluft"),
        161: Seconds("ID_WEB_Zaehler_BetrZeitSolar"),
        162: Voltage("ID_WEB_AnalogOut3"),
        163: Voltage("ID_WEB_AnalogOut4"),
        164: Voltage("ID_WEB_Out_VZU"),
        165: Voltage("ID_WEB_Out_VAB"),
        166: Bool("ID_WEB_Out_VSK"),
        167: Bool("ID_WEB_Out_FRH"),
        168: Voltage("ID_WEB_AnalogIn2"),
        169: Voltage("ID_WEB_AnalogIn3"),
        170: Bool("ID_WEB_SAXin"),
        171: Bool("ID_WEB_SPLin"),
        172: Bool("ID_WEB_Compact_exists"),
        173: Bool("ID_WEB_Durchfluss_WQ"),
        174: Bool("ID_WEB_LIN_exists"),
        175: Celsius("ID_WEB_LIN_ANSAUG_VERDAMPFER"),
        176: Celsius("ID_WEB_LIN_ANSAUG_VERDICHTER"),
        177: Celsius("ID_WEB_LIN_VDH"),
        178: Kelvin("ID_WEB_LIN_UH"),
        179: Kelvin("ID_WEB_LIN_UH_Soll"),
        180: Pressure("ID_WEB_LIN_HD"),
        181: Pressure("ID_WEB_LIN_ND"),
        182: Bool("ID_WEB_LIN_VDH_out"),
        183: Percent2("ID_WEB_HZIO_PWM"),
        184: Speed("ID_WEB_HZIO_VEN"),
        185: Unknown("ID_WEB_HZIO_EVU2"),
        186: Bool("ID_WEB_HZIO_STB"),
        187: Energy("ID_WEB_SEC_Qh_Soll"),
        188: Energy("ID_WEB_SEC_Qh_Ist"),
        189: Celsius("ID_WEB_SEC_TVL_Soll"),
        190: Unknown("ID_WEB_SEC_Software"),
        191: SecOperationMode("ID_WEB_SEC_BZ"),
        192: Unknown("ID_WEB_SEC_VWV"),
        193: Speed("ID_WEB_SEC_VD"),
        194: Celsius("ID_WEB_SEC_VerdEVI"),
        195: Celsius("ID_WEB_SEC_AnsEVI"),
        196: Kelvin("ID_WEB_SEC_UEH_EVI"),
        197: Kelvin("ID_WEB_SEC_UEH_EVI_S"),
        198: Celsius("ID_WEB_SEC_KondTemp"),
        199: Celsius("ID_WEB_SEC_FlussigEx"),
        200: Celsius("ID_WEB_SEC_UK_EEV"),
        201: Pressure("ID_WEB_SEC_EVI_Druck"),
        202: Voltage("ID_WEB_SEC_U_Inv"),
        203: Celsius("ID_WEB_Temperatur_THG_2"),
        204: Celsius("ID_WEB_Temperatur_TWE_2"),
        205: Celsius("ID_WEB_LIN_ANSAUG_VERDAMPFER_2"),
        206: Celsius("ID_WEB_LIN_ANSAUG_VERDICHTER_2"),
        207: Celsius("ID_WEB_LIN_VDH_2"),
        208: Kelvin("ID_WEB_LIN_UH_2"),
        209: Kelvin("ID_WEB_LIN_UH_Soll_2"),
        210: Pressure("ID_WEB_LIN_HD_2"),
        211: Pressure("ID_WEB_LIN_ND_2"),
        212: Bool("ID_WEB_HDin_2"),
        213: Bool("ID_WEB_AVout_2"),
        214: Bool("ID_WEB_VBOout_2"),
        215: Bool("ID_WEB_VD1out_2"),
        216: Bool("ID_WEB_LIN_VDH_out_2"),
        217: SwitchoffFile("ID_WEB_Switchoff2_file_Nr0"),
        218: SwitchoffFile("ID_WEB_Switchoff2_file_Nr1"),
        219: SwitchoffFile("ID_WEB_Switchoff2_file_Nr2"),
        220: SwitchoffFile("ID_WEB_Switchoff2_file_Nr3"),
        221: SwitchoffFile("ID_WEB_Switchoff2_file_Nr4"),
        222: Timestamp("ID_WEB_Switchoff2_file_Time0"),
        223: Timestamp("ID_WEB_Switchoff2_file_Time1"),
        224: Timestamp("ID_WEB_Switchoff2_file_Time2"),
        225: Timestamp("ID_WEB_Switchoff2_file_Time3"),
        226: Timestamp("ID_WEB_Switchoff2_file_Time4"),
        227: Celsius("ID_WEB_RBE_RT_Ist"),
        228: Celsius("ID_WEB_RBE_RT_Soll"),
        229: Celsius("ID_WEB_Temperatur_BW_oben"),
        230: Code_WP("ID_WEB_Code_WP_akt_2"),
        231: Frequency("ID_WEB_Freq_VD"),
        232: Unknown("Unknown_Calculation_232"),
        233: Unknown("Unknown_Calculation_233"),
        234: Unknown("Unknown_Calculation_234"),
        235: Unknown("Unknown_Calculation_235"),
        236: Unknown("Unknown_Calculation_236"),
        237: Unknown("Unknown_Calculation_237"),
        238: Unknown("Unknown_Calculation_238"),
        239: Unknown("Unknown_Calculation_239"),
        240: Unknown("Unknown_Calculation_240"),
        241: Percent2("Circulation_Pump"),
        242: Unknown("Unknown_Calculation_242"),
        243: Unknown("Unknown_Calculation_243"),
        244: Unknown("Unknown_Calculation_244"),
        245: Unknown("Unknown_Calculation_245"),
        246: Unknown("Unknown_Calculation_246"),
        247: Unknown("Unknown_Calculation_247"),
        248: Unknown("Unknown_Calculation_248"),
        249: Unknown("Unknown_Calculation_249"),
        250: Unknown("Unknown_Calculation_250"),
        251: Unknown("Unknown_Calculation_251"),
        252: Unknown("Unknown_Calculation_252"),
        253: Unknown("Unknown_Calculation_253"),
        254: Unknown("Unknown_Calculation_254"),
        255: Unknown("Unknown_Calculation_255"),
        256: Unknown("Unknown_Calculation_256"),
        257: Power("Heat_Output"),
        258: Unknown("Unknown_Calculation_258"),
    }

    def _parse(self, data):
        """Parse raw calculations data."""
        for i, d in enumerate(data):
            c = self.calculations.get(i, False)
            if c is not False and i not in range(81, 91):
                c.value = c._to(d)
                continue
            elif c is not False and i in range(81, 91):
                print(data[i : i + 9])
                c.value = c._to(data[i : i + 9])
                continue
            if c is False and i not in range(81, 91):
                LOGGER.warn(f"Calculation '{i}' not in list of calculationss")

    def _lookup(self, c):
        """Lookup calculation by either id or name."""
        if isinstance(c, int):
            return c, self.calculations.get(c, None)
        if isinstance(c, str):
            try:
                c = int(c)
                return c, self.calculations.get(c, None)
            except ValueError:
                for k, v in self.calculations.items():
                    if v.name == c:
                        return k, v
        LOGGER.warn(f"Calculation '{p}' not found")
        return None, None

    def get(self, c):
        """Get calculation by id or name."""
        id, calculation = self._lookup(c)
        return calculation



import logging

from luxtronik.datatypes import Unknown

logging.basicConfig(level="WARNING")
LOGGER = logging.getLogger("Luxtronik.Visibilities")


class Visibilities:
    """Class that holds all visibilities."""

    visibilities = {
        0: Unknown("ID_Visi_NieAnzeigen"),
        1: Unknown("ID_Visi_ImmerAnzeigen"),
        2: Unknown("ID_Visi_Heizung"),
        3: Unknown("ID_Visi_Brauwasser"),
        4: Unknown("ID_Visi_Schwimmbad"),
        5: Unknown("ID_Visi_Kuhlung"),
        6: Unknown("ID_Visi_Lueftung"),
        7: Unknown("ID_Visi_MK1"),
        8: Unknown("ID_Visi_MK2"),
        9: Unknown("ID_Visi_ThermDesinfekt"),
        10: Unknown("ID_Visi_Zirkulation"),
        11: Unknown("ID_Visi_KuhlTemp_SolltempMK1"),
        12: Unknown("ID_Visi_KuhlTemp_SolltempMK2"),
        13: Unknown("ID_Visi_KuhlTemp_ATDiffMK1"),
        14: Unknown("ID_Visi_KuhlTemp_ATDiffMK2"),
        15: Unknown("ID_Visi_Service_Information"),
        16: Unknown("ID_Visi_Service_Einstellung"),
        17: Unknown("ID_Visi_Service_Sprache"),
        18: Unknown("ID_Visi_Service_DatumUhrzeit"),
        19: Unknown("ID_Visi_Service_Ausheiz"),
        20: Unknown("ID_Visi_Service_Anlagenkonfiguration"),
        21: Unknown("ID_Visi_Service_IBNAssistant"),
        22: Unknown("ID_Visi_Service_ParameterIBNZuruck"),
        23: Unknown("ID_Visi_Temp_Vorlauf"),
        24: Unknown("ID_Visi_Temp_Rucklauf"),
        25: Unknown("ID_Visi_Temp_RL_Soll"),
        26: Unknown("ID_Visi_Temp_Ruecklext"),
        27: Unknown("ID_Visi_Temp_Heissgas"),
        28: Unknown("ID_Visi_Temp_Aussent"),
        29: Unknown("ID_Visi_Temp_BW_Ist"),
        30: Unknown("ID_Visi_Temp_BW_Soll"),
        31: Unknown("ID_Visi_Temp_WQ_Ein"),
        32: Unknown("ID_Visi_Temp_Kaltekreis"),
        33: Unknown("ID_Visi_Temp_MK1_Vorlauf"),
        34: Unknown("ID_Visi_Temp_MK1VL_Soll"),
        35: Unknown("ID_Visi_Temp_Raumstation"),
        36: Unknown("ID_Visi_Temp_MK2_Vorlauf"),
        37: Unknown("ID_Visi_Temp_MK2VL_Soll"),
        38: Unknown("ID_Visi_Temp_Solarkoll"),
        39: Unknown("ID_Visi_Temp_Solarsp"),
        40: Unknown("ID_Visi_Temp_Ext_Energ"),
        41: Unknown("ID_Visi_IN_ASD"),
        42: Unknown("ID_Visi_IN_BWT"),
        43: Unknown("ID_Visi_IN_EVU"),
        44: Unknown("ID_Visi_IN_HD"),
        45: Unknown("ID_Visi_IN_MOT"),
        46: Unknown("ID_Visi_IN_ND"),
        47: Unknown("ID_Visi_IN_PEX"),
        48: Unknown("ID_Visi_IN_SWT"),
        49: Unknown("ID_Visi_OUT_Abtauventil"),
        50: Unknown("ID_Visi_OUT_BUP"),
        51: Unknown("ID_Visi_OUT_FUP1"),
        52: Unknown("ID_Visi_OUT_HUP"),
        53: Unknown("ID_Visi_OUT_Mischer1Auf"),
        54: Unknown("ID_Visi_OUT_Mischer1Zu"),
        55: Unknown("ID_Visi_OUT_Ventilation"),
        56: Unknown("ID_Visi_OUT_Ventil_BOSUP"),
        57: Unknown("ID_Visi_OUT_Verdichter1"),
        58: Unknown("ID_Visi_OUT_Verdichter2"),
        59: Unknown("ID_Visi_OUT_ZIP"),
        60: Unknown("ID_Visi_OUT_ZUP"),
        61: Unknown("ID_Visi_OUT_ZWE1"),
        62: Unknown("ID_Visi_OUT_ZWE2_SST"),
        63: Unknown("ID_Visi_OUT_ZWE3"),
        64: Unknown("ID_Visi_OUT_FUP2"),
        65: Unknown("ID_Visi_OUT_SLP"),
        66: Unknown("ID_Visi_OUT_SUP"),
        67: Unknown("ID_Visi_OUT_Mischer2Auf"),
        68: Unknown("ID_Visi_OUT_Mischer2Zu"),
        69: Unknown("ID_Visi_AblaufZ_WP_Seit"),
        70: Unknown("ID_Visi_AblaufZ_ZWE1_seit"),
        71: Unknown("ID_Visi_AblaufZ_ZWE2_seit"),
        72: Unknown("ID_Visi_AblaufZ_ZWE3_seit"),
        73: Unknown("ID_Visi_AblaufZ_Netzeinv"),
        74: Unknown("ID_Visi_AblaufZ_SSP_Zeit1"),
        75: Unknown("ID_Visi_AblaufZ_VD_Stand"),
        76: Unknown("ID_Visi_AblaufZ_HRM_Zeit"),
        77: Unknown("ID_Visi_AblaufZ_HRW_Zeit"),
        78: Unknown("ID_Visi_AblaufZ_TDI_seit"),
        79: Unknown("ID_Visi_AblaufZ_Sperre_BW"),
        80: Unknown("ID_Visi_Bst_BStdVD1"),
        81: Unknown("ID_Visi_Bst_ImpVD1"),
        82: Unknown("ID_Visi_Bst_dEZVD1"),
        83: Unknown("ID_Visi_Bst_BStdVD2"),
        84: Unknown("ID_Visi_Bst_ImpVD2"),
        85: Unknown("ID_Visi_Bst_dEZVD2"),
        86: Unknown("ID_Visi_Bst_BStdZWE1"),
        87: Unknown("ID_Visi_Bst_BStdZWE2"),
        88: Unknown("ID_Visi_Bst_BStdZWE3"),
        89: Unknown("ID_Visi_Bst_BStdWP"),
        90: Unknown("ID_Visi_Text_Kurzprogramme"),
        91: Unknown("ID_Visi_Text_Zwangsheizung"),
        92: Unknown("ID_Visi_Text_Zwangsbrauchwasser"),
        93: Unknown("ID_Visi_Text_Abtauen"),
        94: Unknown("ID_Visi_EinstTemp_RucklBegr"),
        95: Unknown("ID_Visi_EinstTemp_HystereseHR"),
        96: Unknown("ID_Visi_EinstTemp_TRErhmax"),
        97: Unknown("ID_Visi_EinstTemp_Freig2VD"),
        98: Unknown("ID_Visi_EinstTemp_FreigZWE"),
        99: Unknown("ID_Visi_EinstTemp_Tluftabt"),
        100: Unknown("ID_Visi_EinstTemp_TDISolltemp"),
        101: Unknown("ID_Visi_EinstTemp_HystereseBW"),
        102: Unknown("ID_Visi_EinstTemp_Vorl2VDBW"),
        103: Unknown("ID_Visi_EinstTemp_TAussenmax"),
        104: Unknown("ID_Visi_EinstTemp_TAussenmin"),
        105: Unknown("ID_Visi_EinstTemp_TWQmin"),
        106: Unknown("ID_Visi_EinstTemp_THGmax"),
        107: Unknown("ID_Visi_EinstTemp_TLABTEnde"),
        108: Unknown("ID_Visi_EinstTemp_Absenkbis"),
        109: Unknown("ID_Visi_EinstTemp_Vorlaufmax"),
        110: Unknown("ID_Visi_EinstTemp_TDiffEin"),
        111: Unknown("ID_Visi_EinstTemp_TDiffAus"),
        112: Unknown("ID_Visi_EinstTemp_TDiffmax"),
        113: Unknown("ID_Visi_EinstTemp_TEEHeizung"),
        114: Unknown("ID_Visi_EinstTemp_TEEBrauchw"),
        115: Unknown("ID_Visi_EinstTemp_Vorl2VDSW"),
        116: Unknown("ID_Visi_EinstTemp_VLMaxMk1"),
        117: Unknown("ID_Visi_EinstTemp_VLMaxMk2"),
        118: Unknown("ID_Visi_Priori_Brauchwasser"),
        119: Unknown("ID_Visi_Priori_Heizung"),
        120: Unknown("ID_Visi_Priori_Schwimmbad"),
        121: Unknown("ID_Visi_SysEin_EVUSperre"),
        122: Unknown("ID_Visi_SysEin_Raumstation"),
        123: Unknown("ID_Visi_SysEin_Einbindung"),
        124: Unknown("ID_Visi_SysEin_Mischkreis1"),
        125: Unknown("ID_Visi_SysEin_Mischkreis2"),
        126: Unknown("ID_Visi_SysEin_ZWE1Art"),
        127: Unknown("ID_Visi_SysEin_ZWE1Fkt"),
        128: Unknown("ID_Visi_SysEin_ZWE2Art"),
        129: Unknown("ID_Visi_SysEin_ZWE2Fkt"),
        130: Unknown("ID_Visi_SysEin_ZWE3Art"),
        131: Unknown("ID_Visi_SysEin_ZWE3Fkt"),
        132: Unknown("ID_Visi_SysEin_Stoerung"),
        133: Unknown("ID_Visi_SysEin_Brauchwasser1"),
        134: Unknown("ID_Visi_SysEin_Brauchwasser2"),
        135: Unknown("ID_Visi_SysEin_Brauchwasser3"),
        136: Unknown("ID_Visi_SysEin_Brauchwasser4"),
        137: Unknown("ID_Visi_SysEin_Brauchwasser5"),
        138: Unknown("ID_Visi_SysEin_BWWPmax"),
        139: Unknown("ID_Visi_SysEin_Abtzykmax"),
        140: Unknown("ID_Visi_SysEin_Luftabt"),
        141: Unknown("ID_Visi_SysEin_LuftAbtmax"),
        142: Unknown("ID_Visi_SysEin_Abtauen1"),
        143: Unknown("ID_Visi_SysEin_Abtauen2"),
        144: Unknown("ID_Visi_SysEin_Pumpenoptim"),
        145: Unknown("ID_Visi_SysEin_Zusatzpumpe"),
        146: Unknown("ID_Visi_SysEin_Zugang"),
        147: Unknown("ID_Visi_SysEin_SoledrDurchf"),
        148: Unknown("ID_Visi_SysEin_UberwachungVD"),
        149: Unknown("ID_Visi_SysEin_RegelungHK"),
        150: Unknown("ID_Visi_SysEin_RegelungMK1"),
        151: Unknown("ID_Visi_SysEin_RegelungMK2"),
        152: Unknown("ID_Visi_SysEin_Kuhlung"),
        153: Unknown("ID_Visi_SysEin_Ausheizen"),
        154: Unknown("ID_Visi_SysEin_ElektrAnode"),
        155: Unknown("ID_Visi_SysEin_SWBBer"),
        156: Unknown("ID_Visi_SysEin_SWBMin"),
        157: Unknown("ID_Visi_SysEin_Heizung"),
        158: Unknown("ID_Visi_SysEin_PeriodeMk1"),
        159: Unknown("ID_Visi_SysEin_LaufzeitMk1"),
        160: Unknown("ID_Visi_SysEin_PeriodeMk2"),
        161: Unknown("ID_Visi_SysEin_LaufzeitMk2"),
        162: Unknown("ID_Visi_SysEin_Heizgrenze"),
        163: Unknown("ID_Visi_Enlt_HUP"),
        164: Unknown("ID_Visi_Enlt_ZUP"),
        165: Unknown("ID_Visi_Enlt_BUP"),
        166: Unknown("ID_Visi_Enlt_Ventilator_BOSUP"),
        167: Unknown("ID_Visi_Enlt_MA1"),
        168: Unknown("ID_Visi_Enlt_MZ1"),
        169: Unknown("ID_Visi_Enlt_ZIP"),
        170: Unknown("ID_Visi_Enlt_MA2"),
        171: Unknown("ID_Visi_Enlt_MZ2"),
        172: Unknown("ID_Visi_Enlt_SUP"),
        173: Unknown("ID_Visi_Enlt_SLP"),
        174: Unknown("ID_Visi_Enlt_FP2"),
        175: Unknown("ID_Visi_Enlt_Laufzeit"),
        176: Unknown("ID_Visi_Anlgkonf_Heizung"),
        177: Unknown("ID_Visi_Anlgkonf_Brauchwarmwasser"),
        178: Unknown("ID_Visi_Anlgkonf_Schwimmbad"),
        179: Unknown("ID_Visi_Heizung_Betriebsart"),
        180: Unknown("ID_Visi_Heizung_TemperaturPlusMinus"),
        181: Unknown("ID_Visi_Heizung_Heizkurven"),
        182: Unknown("ID_Visi_Heizung_Zeitschlaltprogramm"),
        183: Unknown("ID_Visi_Heizung_Heizgrenze"),
        184: Unknown("ID_Visi_Mitteltemperatur"),
        185: Unknown("ID_Visi_Dataenlogger"),
        186: Unknown("ID_Visi_Sprachen_DEUTSCH"),
        187: Unknown("ID_Visi_Sprachen_ENGLISH"),
        188: Unknown("ID_Visi_Sprachen_FRANCAIS"),
        189: Unknown("ID_Visi_Sprachen_NORWAY"),
        190: Unknown("ID_Visi_Sprachen_TCHECH"),
        191: Unknown("ID_Visi_Sprachen_ITALIANO"),
        192: Unknown("ID_Visi_Sprachen_NEDERLANDS"),
        193: Unknown("ID_Visi_Sprachen_SVENSKA"),
        194: Unknown("ID_Visi_Sprachen_POLSKI"),
        195: Unknown("ID_Visi_Sprachen_MAGYARUL"),
        196: Unknown("ID_Visi_ErrorUSBspeichern"),
        197: Unknown("ID_Visi_Bst_BStdHz"),
        198: Unknown("ID_Visi_Bst_BStdBW"),
        199: Unknown("ID_Visi_Bst_BStdKue"),
        200: Unknown("ID_Visi_Service_Systemsteuerung"),
        201: Unknown("ID_Visi_Service_Systemsteuerung_Contrast"),
        202: Unknown("ID_Visi_Service_Systemsteuerung_Webserver"),
        203: Unknown("ID_Visi_Service_Systemsteuerung_IPAdresse"),
        204: Unknown("ID_Visi_Service_Systemsteuerung_Fernwartung"),
        205: Unknown("ID_Visi_Paralleleschaltung"),
        206: Unknown("ID_Visi_SysEin_Paralleleschaltung"),
        207: Unknown("ID_Visi_Sprachen_DANSK"),
        208: Unknown("ID_Visi_Sprachen_PORTUGES"),
        209: Unknown("ID_Visi_Heizkurve_Heizung"),
        210: Unknown("ID_Visi_SysEin_Mischkreis3"),
        211: Unknown("ID_Visi_MK3"),
        212: Unknown("ID_Visi_Temp_MK3_Vorlauf"),
        213: Unknown("ID_Visi_Temp_MK3VL_Soll"),
        214: Unknown("ID_Visi_OUT_Mischer3Auf"),
        215: Unknown("ID_Visi_OUT_Mischer3Zu"),
        216: Unknown("ID_Visi_SysEin_RegelungMK3"),
        217: Unknown("ID_Visi_SysEin_PeriodeMk3"),
        218: Unknown("ID_Visi_SysEin_LaufzeitMk3"),
        219: Unknown("ID_Visi_SysEin_Kuhl_Zeit_Ein"),
        220: Unknown("ID_Visi_SysEin_Kuhl_Zeit_Aus"),
        221: Unknown("ID_Visi_AblaufZ_AbtauIn"),
        222: Unknown("ID_Visi_Waermemenge_WS"),
        223: Unknown("ID_Visi_Waermemenge_WQ"),
        224: Unknown("ID_Visi_Enlt_MA3"),
        225: Unknown("ID_Visi_Enlt_MZ3"),
        226: Unknown("ID_Visi_Enlt_FP3"),
        227: Unknown("ID_Visi_OUT_FUP3"),
        228: Unknown("ID_Visi_Temp_Raumstation2"),
        229: Unknown("ID_Visi_Temp_Raumstation3"),
        230: Unknown("ID_Visi_Bst_BStdSW"),
        231: Unknown("ID_Visi_Sprachen_LITAUISCH"),
        232: Unknown("ID_Visi_Sprachen_ESTNICH"),
        233: Unknown("ID_Visi_SysEin_Fernwartung"),
        234: Unknown("ID_Visi_Sprachen_SLOVENISCH"),
        235: Unknown("ID_Visi_EinstTemp_TA_EG"),
        236: Unknown("ID_Visi_Einst_TVLmax_EG"),
        237: Unknown("ID_Visi_SysEin_PoptNachlauf"),
        238: Unknown("ID_Visi_RFV_K_Kuehlin"),
        239: Unknown("ID_Visi_SysEin_EffizienzpumpeNom"),
        240: Unknown("ID_Visi_SysEin_EffizienzpumpeMin"),
        241: Unknown("ID_Visi_SysEin_Effizienzpumpe"),
        242: Unknown("ID_Visi_SysEin_Waermemenge"),
        243: Unknown("ID_Visi_Service_WMZ_Effizienz"),
        244: Unknown("ID_Visi_SysEin_Wm_Versorgung_Korrektur"),
        245: Unknown("ID_Visi_SysEin_Wm_Auswertung_Korrektur"),
        246: Unknown("ID_Visi_IN_AnalogIn"),
        247: Unknown("ID_Visi_Eins_SN_Eingabe"),
        248: Unknown("ID_Visi_OUT_Analog_1"),
        249: Unknown("ID_Visi_OUT_Analog_2"),
        250: Unknown("ID_Visi_Solar"),
        251: Unknown("ID_Visi_SysEin_Solar"),
        252: Unknown("ID_Visi_EinstTemp_TDiffKollmax"),
        253: Unknown("ID_Visi_AblaufZ_HG_Sperre"),
        254: Unknown("ID_Visi_SysEin_Akt_Kuehlung"),
        255: Unknown("ID_Visi_SysEin_Vorlauf_VBO"),
        256: Unknown("ID_Visi_Einst_KRHyst"),
        257: Unknown("ID_Visi_Einst_Akt_Kuehl_Speicher_min"),
        258: Unknown("ID_Visi_Einst_Akt_Kuehl_Freig_WQE"),
        259: Unknown("ID_Visi_SysEin_AbtZykMin"),
        260: Unknown("ID_Visi_SysEin_VD2_Zeit_Min"),
        261: Unknown("ID_Visi_EinstTemp_Hysterese_HR_verkuerzt"),
        262: Unknown("ID_Visi_Einst_Luf_Feuchteschutz_akt"),
        263: Unknown("ID_Visi_Einst_Luf_Reduziert_akt"),
        264: Unknown("ID_Visi_Einst_Luf_Nennlueftung_akt"),
        265: Unknown("ID_Visi_Einst_Luf_Intensivlueftung_akt"),
        266: Unknown("ID_Visi_Temperatur_Lueftung_Zuluft"),
        267: Unknown("ID_Visi_Temperatur_Lueftung_Abluft"),
        268: Unknown("ID_Visi_OUT_Analog_3"),
        269: Unknown("ID_Visi_OUT_Analog_4"),
        270: Unknown("ID_Visi_IN_Analog_2"),
        271: Unknown("ID_Visi_IN_Analog_3"),
        272: Unknown("ID_Visi_IN_SAX"),
        273: Unknown("ID_Visi_OUT_VZU"),
        274: Unknown("ID_Visi_OUT_VAB"),
        275: Unknown("ID_Visi_OUT_VSK"),
        276: Unknown("ID_Visi_OUT_FRH"),
        277: Unknown("ID_Visi_KuhlTemp_SolltempMK3"),
        278: Unknown("ID_Visi_KuhlTemp_ATDiffMK3"),
        279: Unknown("ID_Visi_IN_SPL"),
        280: Unknown("ID_Visi_SysEin_Lueftungsstufen"),
        281: Unknown("ID_Visi_SysEin_Meldung_TDI"),
        282: Unknown("ID_Visi_SysEin_Typ_WZW"),
        283: Unknown("ID_Visi_BACnet"),
        284: Unknown("ID_Visi_Sprachen_SLOWAKISCH"),
        285: Unknown("ID_Visi_Sprachen_LETTISCH"),
        286: Unknown("ID_Visi_Sprachen_FINNISCH"),
        287: Unknown("ID_Visi_Kalibrierung_LWD"),
        288: Unknown("ID_Visi_IN_Durchfluss"),
        289: Unknown("ID_Visi_LIN_ANSAUG_VERDICHTER"),
        290: Unknown("ID_Visi_LIN_VDH"),
        291: Unknown("ID_Visi_LIN_UH"),
        292: Unknown("ID_Visi_LIN_Druck"),
        293: Unknown("ID_Visi_Einst_Sollwert_TRL_Kuehlen"),
        294: Unknown("ID_Visi_Entl_ExVentil"),
        295: Unknown("ID_Visi_Einst_Medium_Waermequelle"),
        296: Unknown("ID_Visi_Einst_Multispeicher"),
        297: Unknown("ID_Visi_Einst_Minimale_Ruecklaufsolltemperatur"),
        298: Unknown("ID_Visi_Einst_PKuehlTime"),
        299: Unknown("ID_Visi_Sprachen_TUERKISCH"),
        300: Unknown("ID_Visi_RBE"),
        301: Unknown("ID_Visi_Einst_Luf_Stufen_Faktor"),
        302: Unknown("ID_Visi_Freigabe_Zeit_ZWE"),
        303: Unknown("ID_Visi_Einst_min_VL_Kuehl"),
        304: Unknown("ID_Visi_ZWE1"),
        305: Unknown("ID_Visi_ZWE2"),
        306: Unknown("ID_Visi_ZWE3"),
        307: Unknown("ID_Visi_SEC"),
        308: Unknown("ID_Visi_HZIO"),
        309: Unknown("ID_Visi_WPIO"),
        310: Unknown("ID_Visi_LIN_ANSAUG_VERDAMPFER"),
        311: Unknown("ID_Visi_LIN_MULTI1"),
        312: Unknown("ID_Visi_LIN_MULTI2"),
        313: Unknown("ID_Visi_Einst_Leistung_ZWE"),
        314: Unknown("ID_Visi_Sprachen_ESPANOL"),
        315: Unknown("ID_Visi_Temp_BW_oben"),
        316: Unknown("ID_Visi_MAXIO"),
        317: Unknown("ID_Visi_OUT_Abtauwunsch"),
        318: Unknown("ID_Visi_SmartGrid"),
        319: Unknown("ID_Visi_Drehzahlgeregelt"),
        320: Unknown("ID_Visi_P155_Inverter"),
        321: Unknown("ID_Visi_Leistungsfreigabe"),
        322: Unknown("ID_Visi_Einst_Vorl_akt_Kuehl"),
        323: Unknown("ID_Visi_Einst_Abtauen_im_Warmwasser"),
        324: Unknown("ID_Visi_Waermemenge_ZWE"),
        325: Unknown("Unknown_Visibility_325"),
        326: Unknown("Unknown_Visibility_326"),
        327: Unknown("Unknown_Visibility_327"),
        328: Unknown("Unknown_Visibility_328"),
        329: Unknown("Unknown_Visibility_329"),
        330: Unknown("Unknown_Visibility_330"),
        331: Unknown("Unknown_Visibility_331"),
        332: Unknown("Unknown_Visibility_332"),
        333: Unknown("Unknown_Visibility_333"),
        334: Unknown("Unknown_Visibility_334"),
        335: Unknown("Unknown_Visibility_335"),
        336: Unknown("Unknown_Visibility_336"),
        337: Unknown("Unknown_Visibility_337"),
        338: Unknown("Unknown_Visibility_338"),
        339: Unknown("Unknown_Visibility_339"),
        340: Unknown("Unknown_Visibility_340"),
        341: Unknown("Unknown_Visibility_341"),
        342: Unknown("Unknown_Visibility_342"),
        343: Unknown("Unknown_Visibility_343"),
        344: Unknown("Unknown_Visibility_344"),
        345: Unknown("Unknown_Visibility_345"),
    }

    def _parse(self, data):
        """Parse raw visibility data."""
        for i, d in enumerate(data):
            v = self.visibilities.get(i, False)
            if v is not False:
                v.value = v._to(d)
            else:
                LOGGER.warn(f"Visibility '{i}' not in list of visibilities")

    def _lookup(self, v):
        """Lookup visibility by either id or name."""
        if isinstance(v, int):
            return v, self.visibilities.get(v, None)
        if isinstance(v, str):
            try:
                v = int(v)
                return v, self.visibilities.get(v, None)
            except ValueError:
                for k, v in self.visibilities.items():
                    if v.name == v:
                        return k, v
        LOGGER.warn(f"Visibility '{p}' not found")
        return None, None

    def get(self, v):
        """Get visibility by id or name."""
        id, visibility = self._lookup(v)
        return visibility
